@@ -75,12 +75,12 @@
 #include "base/profile.h"
 #include "base/rss/rss_autodownloader.h"
 #include "base/rss/rss_session.h"
-#include "base/scanfoldersmodel.h"
 #include "base/search/searchpluginmanager.h"
 #include "base/settingsstorage.h"
+#include "base/torrentfileswatcher.h"
+#include "base/utils/compare.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
-#include "base/utils/string.h"
 #include "base/version.h"
 #include "applicationinstancemanager.h"
 #include "filelogger.h"
@@ -100,22 +100,10 @@
 namespace
 {
 #define SETTINGS_KEY(name) "Application/" name
-
-    // FileLogger properties keys
-#define FILELOGGER_SETTINGS_KEY(name) QStringLiteral(SETTINGS_KEY("FileLogger/") name)
-    const QString KEY_FILELOGGER_ENABLED = FILELOGGER_SETTINGS_KEY("Enabled");
-    const QString KEY_FILELOGGER_PATH = FILELOGGER_SETTINGS_KEY("Path");
-    const QString KEY_FILELOGGER_BACKUP = FILELOGGER_SETTINGS_KEY("Backup");
-    const QString KEY_FILELOGGER_DELETEOLD = FILELOGGER_SETTINGS_KEY("DeleteOld");
-    const QString KEY_FILELOGGER_MAXSIZEBYTES = FILELOGGER_SETTINGS_KEY("MaxSizeBytes");
-    const QString KEY_FILELOGGER_AGE = FILELOGGER_SETTINGS_KEY("Age");
-    const QString KEY_FILELOGGER_AGETYPE = FILELOGGER_SETTINGS_KEY("AgeType");
-
-    // just a shortcut
-    inline SettingsStorage *settings() { return  SettingsStorage::instance(); }
+#define FILELOGGER_SETTINGS_KEY(name) (SETTINGS_KEY("FileLogger/") name)
 
     const QString LOG_FOLDER = QStringLiteral("logs");
-    const QChar PARAMS_SEPARATOR = '|';
+    const QChar PARAMS_SEPARATOR = QLatin1Char('|');
 
     const QString DEFAULT_PORTABLE_MODE_PROFILE_DIR = QStringLiteral("profile");
 
@@ -133,6 +121,13 @@ Application::Application(int &argc, char **argv)
     , m_running(false)
     , m_shutdownAct(ShutdownDialogAction::Exit)
     , m_commandLineArgs(parseCommandLine(this->arguments()))
+    , m_storeFileLoggerEnabled(FILELOGGER_SETTINGS_KEY("Enabled"))
+    , m_storeFileLoggerBackup(FILELOGGER_SETTINGS_KEY("Backup"))
+    , m_storeFileLoggerDeleteOld(FILELOGGER_SETTINGS_KEY("DeleteOld"))
+    , m_storeFileLoggerMaxSize(FILELOGGER_SETTINGS_KEY("MaxSizeBytes"))
+    , m_storeFileLoggerAge(FILELOGGER_SETTINGS_KEY("Age"))
+    , m_storeFileLoggerAgeType(FILELOGGER_SETTINGS_KEY("AgeType"))
+    , m_storeFileLoggerPath(FILELOGGER_SETTINGS_KEY("Path"))
 {
     qRegisterMetaType<Log::Msg>("Log::Msg");
     qRegisterMetaType<Log::Peer>("Log::Peer");
@@ -141,7 +136,9 @@ Application::Application(int &argc, char **argv)
     setOrganizationDomain("qbittorrent.org");
 #if !defined(DISABLE_GUI)
     setDesktopFileName("org.qbittorrent.qBittorrent");
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
     setAttribute(Qt::AA_UseHighDpiPixmaps, true);  // opt-in to the high DPI pixmap support
+#endif
     setQuitOnLastWindowClosed(false);
     QPixmapCache::setCacheLimit(PIXMAP_CACHE_SIZE);
 #endif
@@ -152,17 +149,11 @@ Application::Application(int &argc, char **argv)
     const QString profileDir = portableModeEnabled
         ? QDir(QCoreApplication::applicationDirPath()).absoluteFilePath(DEFAULT_PORTABLE_MODE_PROFILE_DIR)
         : m_commandLineArgs.profileDir;
-#ifdef Q_OS_WIN
-    const QString instanceId = (profileDir + (m_commandLineArgs.configurationName.isEmpty() ? QString {} : ('/' + m_commandLineArgs.configurationName))).toLower();
-#else
-    const QString instanceId = profileDir + (m_commandLineArgs.configurationName.isEmpty() ? QString {} : ('/' + m_commandLineArgs.configurationName));
-#endif
-    const QString appId = QLatin1String("qBittorrent-") + Utils::Misc::getUserIDString() + '@' + instanceId;
-    m_instanceManager = new ApplicationInstanceManager {appId, this};
-
     Profile::initInstance(profileDir, m_commandLineArgs.configurationName,
                         (m_commandLineArgs.relativeFastresumePaths || portableModeEnabled));
 
+    m_instanceManager = new ApplicationInstanceManager {Profile::instance()->location(SpecialFolder::Config), this};
+
     Logger::initInstance();
     SettingsStorage::initInstance();
     Preferences::initInstance();
@@ -215,7 +206,7 @@ const QBtCommandLineParameters &Application::commandLineArgs() const
 
 bool Application::isFileLoggerEnabled() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_ENABLED, true);
+    return m_storeFileLoggerEnabled.get(true);
 }
 
 void Application::setFileLoggerEnabled(const bool value)
@@ -224,49 +215,48 @@ void Application::setFileLoggerEnabled(const bool value)
         m_fileLogger = new FileLogger(fileLoggerPath(), isFileLoggerBackup(), fileLoggerMaxSize(), isFileLoggerDeleteOld(), fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
     else if (!value)
         delete m_fileLogger;
-    settings()->storeValue(KEY_FILELOGGER_ENABLED, value);
+    m_storeFileLoggerEnabled = value;
 }
 
 QString Application::fileLoggerPath() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_PATH
-        , QString {specialFolderLocation(SpecialFolder::Data) + LOG_FOLDER});
+    return m_storeFileLoggerPath.get(QDir(specialFolderLocation(SpecialFolder::Data)).absoluteFilePath(LOG_FOLDER));
 }
 
 void Application::setFileLoggerPath(const QString &path)
 {
     if (m_fileLogger)
         m_fileLogger->changePath(path);
-    settings()->storeValue(KEY_FILELOGGER_PATH, path);
+    m_storeFileLoggerPath = path;
 }
 
 bool Application::isFileLoggerBackup() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_BACKUP, true);
+    return m_storeFileLoggerBackup.get(true);
 }
 
 void Application::setFileLoggerBackup(const bool value)
 {
     if (m_fileLogger)
         m_fileLogger->setBackup(value);
-    settings()->storeValue(KEY_FILELOGGER_BACKUP, value);
+    m_storeFileLoggerBackup = value;
 }
 
 bool Application::isFileLoggerDeleteOld() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_DELETEOLD, true);
+    return m_storeFileLoggerDeleteOld.get(true);
 }
 
 void Application::setFileLoggerDeleteOld(const bool value)
 {
     if (value && m_fileLogger)
         m_fileLogger->deleteOld(fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
-    settings()->storeValue(KEY_FILELOGGER_DELETEOLD, value);
+    m_storeFileLoggerDeleteOld = value;
 }
 
 int Application::fileLoggerMaxSize() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_MAXSIZEBYTES, DEFAULT_FILELOG_SIZE);
+    const int val = m_storeFileLoggerMaxSize.get(DEFAULT_FILELOG_SIZE);
     return std::min(std::max(val, MIN_FILELOG_SIZE), MAX_FILELOG_SIZE);
 }
 
@@ -275,34 +265,34 @@ void Application::setFileLoggerMaxSize(const int bytes)
     const int clampedValue = std::min(std::max(bytes, MIN_FILELOG_SIZE), MAX_FILELOG_SIZE);
     if (m_fileLogger)
         m_fileLogger->setMaxSize(clampedValue);
-    settings()->storeValue(KEY_FILELOGGER_MAXSIZEBYTES, clampedValue);
+    m_storeFileLoggerMaxSize = clampedValue;
 }
 
 int Application::fileLoggerAge() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_AGE, 1);
+    const int val = m_storeFileLoggerAge.get(1);
     return std::min(std::max(val, 1), 365);
 }
 
 void Application::setFileLoggerAge(const int value)
 {
-    settings()->storeValue(KEY_FILELOGGER_AGE, std::min(std::max(value, 1), 365));
+    m_storeFileLoggerAge = std::min(std::max(value, 1), 365);
 }
 
 int Application::fileLoggerAgeType() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_AGETYPE, 1);
+    const int val = m_storeFileLoggerAgeType.get(1);
     return ((val < 0) || (val > 2)) ? 1 : val;
 }
 
 void Application::setFileLoggerAgeType(const int value)
 {
-    settings()->storeValue(KEY_FILELOGGER_AGETYPE, ((value < 0) || (value > 2)) ? 1 : value);
+    m_storeFileLoggerAgeType = ((value < 0) || (value > 2)) ? 1 : value;
 }
 
 void Application::processMessage(const QString &message)
 {
-    const QStringList params = message.split(PARAMS_SEPARATOR, QString::SkipEmptyParts);
+    const QStringList params = message.split(PARAMS_SEPARATOR, Qt::SkipEmptyParts);
     // If Application is not running (i.e., other
     // components are not ready) store params
     if (m_running)
@@ -313,14 +303,6 @@ void Application::processMessage(const QString &message)
 
 void Application::runExternalProgram(const BitTorrent::Torrent *torrent) const
 {
-    QString program = Preferences::instance()->getAutoRunProgram().trimmed();
-    program.replace("%N", torrent->name());
-    program.replace("%L", torrent->category());
-
-    QStringList tags = torrent->tags().values();
-    std::sort(tags.begin(), tags.end(), Utils::String::naturalLessThan<Qt::CaseInsensitive>);
-    program.replace("%G", tags.join(','));
-
 #if defined(Q_OS_WIN)
     const auto chopPathSep = [](const QString &str) -> QString
     {
@@ -328,21 +310,76 @@ void Application::runExternalProgram(const BitTorrent::Torrent *torrent) const
             return str.mid(0, (str.length() -1));
         return str;
     };
-    program.replace("%F", chopPathSep(Utils::Fs::toNativePath(torrent->contentPath())));
-    program.replace("%R", chopPathSep(Utils::Fs::toNativePath(torrent->rootPath())));
-    program.replace("%D", chopPathSep(Utils::Fs::toNativePath(torrent->savePath())));
-#else
-    program.replace("%F", Utils::Fs::toNativePath(torrent->contentPath()));
-    program.replace("%R", Utils::Fs::toNativePath(torrent->rootPath()));
-    program.replace("%D", Utils::Fs::toNativePath(torrent->savePath()));
 #endif
-    program.replace("%C", QString::number(torrent->filesCount()));
-    program.replace("%Z", QString::number(torrent->totalSize()));
-    program.replace("%T", torrent->currentTracker());
-    program.replace("%I", torrent->hash());
 
-    Logger *logger = Logger::instance();
-    logger->addMessage(tr("Torrent: %1, running external program, command: %2").arg(torrent->name(), program));
+    QString program = Preferences::instance()->getAutoRunProgram().trimmed();
+
+    for (int i = (program.length() - 2); i >= 0; --i)
+    {
+        if (program[i] != QLatin1Char('%'))
+            continue;
+
+        const ushort specifier = program[i + 1].unicode();
+        switch (specifier)
+        {
+        case u'C':
+            program.replace(i, 2, QString::number(torrent->filesCount()));
+            break;
+        case u'D':
+#if defined(Q_OS_WIN)
+            program.replace(i, 2, chopPathSep(Utils::Fs::toNativePath(torrent->savePath())));
+#else
+            program.replace(i, 2, Utils::Fs::toNativePath(torrent->savePath()));
+#endif
+            break;
+        case u'F':
+#if defined(Q_OS_WIN)
+            program.replace(i, 2, chopPathSep(Utils::Fs::toNativePath(torrent->contentPath())));
+#else
+            program.replace(i, 2, Utils::Fs::toNativePath(torrent->contentPath()));
+#endif
+            break;
+        case u'G':
+            program.replace(i, 2, torrent->tags().join(QLatin1String(",")));
+            break;
+        case u'I':
+            program.replace(i, 2, (torrent->infoHash().v1().isValid() ? torrent->infoHash().v1().toString() : QLatin1String("-")));
+            break;
+        case u'J':
+            program.replace(i, 2, (torrent->infoHash().v2().isValid() ? torrent->infoHash().v2().toString() : QLatin1String("-")));
+            break;
+        case u'K':
+            program.replace(i, 2, torrent->id().toString());
+            break;
+        case u'L':
+            program.replace(i, 2, torrent->category());
+            break;
+        case u'N':
+            program.replace(i, 2, torrent->name());
+            break;
+        case u'R':
+#if defined(Q_OS_WIN)
+            program.replace(i, 2, chopPathSep(Utils::Fs::toNativePath(torrent->rootPath())));
+#else
+            program.replace(i, 2, Utils::Fs::toNativePath(torrent->rootPath()));
+#endif
+            break;
+        case u'T':
+            program.replace(i, 2, torrent->currentTracker());
+            break;
+        case u'Z':
+            program.replace(i, 2, QString::number(torrent->totalSize()));
+            break;
+        default:
+            // do nothing
+            break;
+        }
+
+        // decrement `i` to avoid unwanted replacement, example pattern: "%%N"
+        --i;
+    }
+
+    LogMsg(tr("Torrent: %1, running external program, command: %2").arg(torrent->name(), program));
 
 #if defined(Q_OS_WIN)
     auto programWchar = std::make_unique<wchar_t[]>(program.length() + 1);
@@ -358,7 +395,6 @@ void Application::runExternalProgram(const BitTorrent::Torrent *torrent) const
     for (int i = 1; i < argCount; ++i)
         argList += QString::fromWCharArray(args[i]);
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
     QProcess proc;
     proc.setProgram(QString::fromWCharArray(args[0]));
     proc.setArguments(argList);
@@ -384,24 +420,17 @@ void Application::runExternalProgram(const BitTorrent::Torrent *torrent) const
         args->startupInfo->hStdError = nullptr;
     });
     proc.startDetached();
-#else
-    QProcess::startDetached(QString::fromWCharArray(args[0]), argList);
-#endif // QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
 #else // Q_OS_WIN
     // Cannot give users shell environment by default, as doing so could
     // enable command injection via torrent name and other arguments
     // (especially when some automated download mechanism has been setup).
     // See: https://github.com/qbittorrent/qBittorrent/issues/10925
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
     QStringList args = QProcess::splitCommand(program);
     if (args.isEmpty())
         return;
 
     const QString command = args.takeFirst();
     QProcess::startDetached(command, args);
-#else
-    QProcess::startDetached(program);
-#endif
 #endif
 }
 
@@ -522,7 +551,7 @@ void Application::processParams(const QStringList &params)
 
         if (param.startsWith(QLatin1String("@addPaused=")))
         {
-            torrentParams.addPaused = (param.midRef(11).toInt() != 0);
+            torrentParams.addPaused = (QStringView(param).mid(11).toInt() != 0);
             continue;
         }
 
@@ -552,7 +581,7 @@ void Application::processParams(const QStringList &params)
 
         if (param.startsWith(QLatin1String("@skipDialog=")))
         {
-            skipTorrentDialog = (param.midRef(12).toInt() != 0);
+            skipTorrentDialog = (QStringView(param).mid(12).toInt() != 0);
             continue;
         }
 
@@ -584,7 +613,7 @@ int Application::exec(const QStringList &params)
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::allTorrentsFinished, this, &Application::allTorrentsFinished, Qt::QueuedConnection);
 
         Net::GeoIPManager::initInstance();
-        ScanFoldersModel::initInstance();
+        TorrentFilesWatcher::initInstance();
 
 #ifndef DISABLE_WEBUI
         m_webui = new WebUI;
@@ -601,7 +630,7 @@ int Application::exec(const QStringList &params)
     catch (const RuntimeError &err)
     {
 #ifdef DISABLE_GUI
-        fprintf(stderr, "%s", err.what());
+        fprintf(stderr, "%s", qPrintable(err.message()));
 #else
         QMessageBox msgBox;
         msgBox.setIcon(QMessageBox::Critical);
@@ -616,18 +645,19 @@ int Application::exec(const QStringList &params)
 
 #ifdef DISABLE_GUI
 #ifndef DISABLE_WEBUI
-    Preferences *const pref = Preferences::instance();
-    // Display some information to the user
+    const Preferences *pref = Preferences::instance();
+
+    const auto scheme = QString::fromLatin1(pref->isWebUiHttpsEnabled() ? "https" : "http");
+    const auto url = QString::fromLatin1("%1://localhost:%2\n").arg(scheme, QString::number(pref->getWebUiPort()));
     const QString mesg = QString::fromLatin1("\n******** %1 ********\n").arg(tr("Information"))
-        + tr("To control qBittorrent, access the Web UI at %1")
-            .arg(QString("http://localhost:") + QString::number(pref->getWebUiPort())) + '\n';
-    printf("%s", qUtf8Printable(mesg));
+        + tr("To control qBittorrent, access the WebUI at: %1").arg(url);
+    printf("%s\n", qUtf8Printable(mesg));
 
     if (pref->getWebUIPassword() == "ARQ77eY1NUZaQsuDHbIMCA==:0WMRkYTUWVT9wVvdDtHAjU9b3b7uB8NR1Gur2hmQCvCDpm39Q+PsJRJPaCU51dEiz+dTzh8qbPsL8WkFljQYFQ==")
     {
         const QString warning = tr("The Web UI administrator username is: %1").arg(pref->getWebUiUsername()) + '\n'
-            + tr("The Web UI administrator password is still the default one: %1").arg("adminadmin") + '\n'
-            + tr("This is a security risk, please consider changing your password from program preferences.") + '\n';
+            + tr("The Web UI administrator password has not been changed from the default: %1").arg("adminadmin") + '\n'
+            + tr("This is a security risk, please change your password in program preferences.") + '\n';
         printf("%s", qUtf8Printable(warning));
     }
 #endif // DISABLE_WEBUI
@@ -780,7 +810,7 @@ void Application::cleanup()
     delete RSS::AutoDownloader::instance();
     delete RSS::Session::instance();
 
-    ScanFoldersModel::freeInstance();
+    TorrentFilesWatcher::freeInstance();
     BitTorrent::Session::freeInstance();
     Net::GeoIPManager::freeInstance();
     Net::DownloadManager::freeInstance();
