@@ -134,7 +134,7 @@ int main(int argc, char *argv[])
     // We must save it here because QApplication constructor may change it
     bool isOneArg = (argc == 2);
 
-#if !defined(DISABLE_GUI) && (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0)) && !defined(DISABLE_GUI)
     // Attribute Qt::AA_EnableHighDpiScaling must be set before QCoreApplication is created
     if (qgetenv("QT_ENABLE_HIGHDPI_SCALING").isEmpty() && qgetenv("QT_AUTO_SCREEN_SCALE_FACTOR").isEmpty())
         Application::setAttribute(Qt::AA_EnableHighDpiScaling, true);
@@ -188,7 +188,6 @@ int main(int argc, char *argv[])
 #ifndef DISABLE_GUI
             if (!userAgreesWithLegalNotice())
                 return EXIT_SUCCESS;
-
 #elif defined(Q_OS_WIN)
             if (_isatty(_fileno(stdin))
                 && _isatty(_fileno(stdout))
@@ -201,6 +200,8 @@ int main(int argc, char *argv[])
                 && !userAgreesWithLegalNotice())
                 return EXIT_SUCCESS;
 #endif
+
+            setCurrentMigrationVersion();
         }
 
         // Check if qBittorrent is already running for this user
@@ -235,7 +236,7 @@ int main(int argc, char *argv[])
         // 3. https://bugreports.qt.io/browse/QTBUG-46015
 
         qputenv("QT_BEARER_POLL_TIMEOUT", QByteArray::number(-1));
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)) && !defined(DISABLE_GUI)
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0)) && !defined(DISABLE_GUI)
         // this is the default in Qt6
         app->setAttribute(Qt::AA_DisableWindowContextHelpButton);
 #endif
@@ -251,6 +252,9 @@ int main(int argc, char *argv[])
 
         // On OS X the standard is to not show icons in the menus
         app->setAttribute(Qt::AA_DontShowIconsInMenus);
+#else
+        if (!Preferences::instance()->iconsInMenusEnabled())
+            app->setAttribute(Qt::AA_DontShowIconsInMenus);
 #endif
 
         if (!firstTimeUser)
@@ -308,7 +312,7 @@ int main(int argc, char *argv[])
     }
     catch (const CommandLineParameterError &er)
     {
-        displayBadArgMessage(er.messageForUser());
+        displayBadArgMessage(er.message());
         return EXIT_FAILURE;
     }
 }
@@ -378,11 +382,7 @@ void showSplashScreen()
     const QString version = QBT_VERSION;
     painter.setPen(QPen(Qt::white));
     painter.setFont(QFont("Arial", 22, QFont::Black));
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
     painter.drawText(224 - painter.fontMetrics().horizontalAdvance(version), 270, version);
-#else
-    painter.drawText(224 - painter.fontMetrics().width(version), 270, version);
-#endif
     QSplashScreen *splash = new QSplashScreen(splashImg);
     splash->show();
     QTimer::singleShot(1500, splash, &QObject::deleteLater);
