@@ -32,18 +32,21 @@
 
 using namespace BitTorrent;
 
-PeerAddress PeerAddress::parse(const QString &address)
+PeerAddress PeerAddress::parse(const QStringView address)
 {
-    QVector<QStringRef> ipPort;
+    QList<QStringView> ipPort;
 
-    if (address.startsWith('[') && address.contains("]:")) {  // IPv6
-        ipPort = address.splitRef("]:");
+    if (address.startsWith(u'[') && address.contains(QLatin1String("]:")))
+    {  // IPv6
+        ipPort = address.split(QString::fromLatin1("]:"));
         ipPort[0] = ipPort[0].mid(1);  // chop '['
     }
-    else if (address.contains(':')) {  // IPv4
-        ipPort = address.splitRef(':');
+    else if (address.contains(u':'))
+    {  // IPv4
+        ipPort = address.split(u':');
     }
-    else {
+    else
+    {
         return {};
     }
 
