@@ -33,8 +33,8 @@
 #include <QCoreApplication>
 #include <QtContainerFwd>
 
+#include "base/3rdparty/expected.hpp"
 #include "base/indexrange.h"
-#include "abstractfilestorage.h"
 #include "torrentcontentlayout.h"
 
 class QByteArray;
@@ -45,39 +45,39 @@ class QUrl;
 namespace BitTorrent
 {
     class InfoHash;
-    class TrackerEntry;
+    struct TrackerEntry;
 
-    class TorrentInfo final : public AbstractFileStorage
+    class TorrentInfo
     {
         Q_DECLARE_TR_FUNCTIONS(TorrentInfo)
 
     public:
-        explicit TorrentInfo(std::shared_ptr<const lt::torrent_info> nativeInfo = {});
+        TorrentInfo() = default;
         TorrentInfo(const TorrentInfo &other);
 
-        static TorrentInfo load(const QByteArray &data, QString *error = nullptr) noexcept;
-        static TorrentInfo loadFromFile(const QString &path, QString *error = nullptr) noexcept;
-        void saveToFile(const QString &path) const;
+        explicit TorrentInfo(const lt::torrent_info &nativeInfo);
+
+        static nonstd::expected<TorrentInfo, QString> load(const QByteArray &data) noexcept;
+        static nonstd::expected<TorrentInfo, QString> loadFromFile(const QString &path) noexcept;
+        nonstd::expected<void, QString> saveToFile(const QString &path) const;
 
         TorrentInfo &operator=(const TorrentInfo &other);
 
         bool isValid() const;
-        InfoHash hash() const;
+        InfoHash infoHash() const;
         QString name() const;
         QDateTime creationDate() const;
         QString creator() const;
         QString comment() const;
         bool isPrivate() const;
         qlonglong totalSize() const;
-        int filesCount() const override;
+        int filesCount() const;
         int pieceLength() const;
         int pieceLength(int index) const;
         int piecesCount() const;
-        QString filePath(int index) const override;
+        QString filePath(int index) const;
         QStringList filePaths() const;
-        QString fileName(int index) const override;
-        QString origFilePath(int index) const;
-        qlonglong fileSize(int index) const override;
+        qlonglong fileSize(int index) const;
         qlonglong fileOffset(int index) const;
         QVector<TrackerEntry> trackers() const;
         QVector<QUrl> urlSeeds() const;
@@ -92,21 +92,20 @@ namespace BitTorrent
         PieceRange filePieces(const QString &file) const;
         PieceRange filePieces(int fileIndex) const;
 
-        void renameFile(int index, const QString &newPath) override;
-
-        QString rootFolder() const;
-        bool hasRootFolder() const;
-        void setContentLayout(TorrentContentLayout layout);
-
         std::shared_ptr<lt::torrent_info> nativeInfo() const;
+        QVector<lt::file_index_t> nativeIndexes() const;
 
     private:
         // returns file index or -1 if fileName is not found
         int fileIndex(const QString &fileName) const;
-        void stripRootFolder();
-        void addRootFolder();
-        TorrentContentLayout defaultContentLayout() const;
+        TorrentContentLayout contentLayout() const;
 
-        std::shared_ptr<lt::torrent_info> m_nativeInfo;
+        std::shared_ptr<const lt::torrent_info> m_nativeInfo;
+
+        // internal indexes of files (payload only, excluding any .pad files)
+        // by which they are addressed in libtorrent
+        QVector<lt::file_index_t> m_nativeIndexes;
     };
 }
+
+Q_DECLARE_METATYPE(BitTorrent::TorrentInfo)
