@@ -29,8 +29,9 @@
 
 #pragma once
 
+#include <QObject>
 #include <QtContainerFwd>
-#include <QVariant>
+#include <QtGlobal>
 
 #include "base/utils/net.h"
 
@@ -39,50 +40,59 @@ class QNetworkCookie;
 class QSize;
 class QTime;
 
-enum SchedulerDays
+namespace Scheduler
 {
-    EVERY_DAY,
-    WEEK_DAYS,
-    WEEK_ENDS,
-    MON,
-    TUE,
-    WED,
-    THU,
-    FRI,
-    SAT,
-    SUN
-};
+    Q_NAMESPACE
 
-namespace TrayIcon
-{
-    enum Style
+    enum class Days : int
     {
-        NORMAL = 0,
-        MONO_DARK,
-        MONO_LIGHT
+        EveryDay = 0,
+        Weekday = 1,
+        Weekend = 2,
+        Monday = 3,
+        Tuesday = 4,
+        Wednesday = 5,
+        Thursday = 6,
+        Friday = 7,
+        Saturday = 8,
+        Sunday = 9
     };
+    Q_ENUM_NS(Days)
 }
 
 namespace DNS
 {
-    enum Service
+    Q_NAMESPACE
+
+    enum class Service : int
     {
-        DYNDNS,
-        NOIP,
-        NONE = -1
+        DynDNS = 0,
+        NoIP = 1,
+        None = -1
     };
+    Q_ENUM_NS(Service)
+}
+
+namespace TrayIcon
+{
+    Q_NAMESPACE
+
+    enum class Style : int
+    {
+        Normal = 0,
+        MonoDark = 1,
+        MonoLight = 2
+    };
+    Q_ENUM_NS(Style)
 }
 
 class Preferences : public QObject
 {
     Q_OBJECT
-    Q_DISABLE_COPY(Preferences)
+    Q_DISABLE_COPY_MOVE(Preferences)
 
     Preferences();
 
-    const QVariant value(const QString &key, const QVariant &defaultValue = {}) const;
-    void setValue(const QString &key, const QVariant &value);
-
     static Preferences *m_instance;
 
 signals:
@@ -132,8 +142,6 @@ public:
     // Downloads
     QString lastLocationPath() const;
     void setLastLocationPath(const QString &path);
-    QVariantHash getScanDirs() const;
-    void setScanDirs(const QVariantHash &dirs);
     QString getScanDirsLastPath() const;
     void setScanDirsLastPath(const QString &path);
     bool isMailNotificationEnabled() const;
@@ -162,8 +170,8 @@ public:
     void setSchedulerStartTime(const QTime &time);
     QTime getSchedulerEndTime() const;
     void setSchedulerEndTime(const QTime &time);
-    SchedulerDays getSchedulerDays() const;
-    void setSchedulerDays(SchedulerDays days);
+    Scheduler::Days getSchedulerDays() const;
+    void setSchedulerDays(Scheduler::Days days);
 
     // Search
     bool isSearchEnabled() const;
@@ -227,11 +235,17 @@ public:
     QString getWebUICustomHTTPHeaders() const;
     void setWebUICustomHTTPHeaders(const QString &headers);
 
+    // Reverse proxy
+    bool isWebUIReverseProxySupportEnabled() const;
+    void setWebUIReverseProxySupportEnabled(bool enabled);
+    QString getWebUITrustedReverseProxiesList() const;
+    void setWebUITrustedReverseProxiesList(const QString &addr);
+
     // Dynamic DNS
     bool isDynDNSEnabled() const;
     void setDynDNSEnabled(bool enabled);
     DNS::Service getDynDNSService() const;
-    void setDynDNSService(int service);
+    void setDynDNSService(DNS::Service service);
     QString getDynDomainName() const;
     void setDynDomainName(const QString &name);
     QString getDynDNSUsername() const;
@@ -248,7 +262,7 @@ public:
     void setAutoRunEnabled(bool enabled);
     QString getAutoRunProgram() const;
     void setAutoRunProgram(const QString &program);
-#if defined(Q_OS_WIN) && (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+#if defined(Q_OS_WIN)
     bool isAutoRunConsoleEnabled() const;
     void setAutoRunConsoleEnabled(bool enabled);
 #endif
@@ -301,8 +315,8 @@ public:
     bool confirmRemoveAllTags() const;
     void setConfirmRemoveAllTags(bool enabled);
 #ifndef Q_OS_MACOS
-    bool systrayIntegration() const;
-    void setSystrayIntegration(bool enabled);
+    bool systemTrayEnabled() const;
+    void setSystemTrayEnabled(bool enabled);
     bool minimizeToTrayNotified() const;
     void setMinimizeToTrayNotified(bool b);
     bool minimizeToTray() const;
@@ -313,6 +327,8 @@ public:
     void setCloseToTrayNotified(bool b);
     TrayIcon::Style trayIconStyle() const;
     void setTrayIconStyle(TrayIcon::Style style);
+    bool iconsInMenusEnabled() const;
+    void setIconsInMenusEnabled(bool enable);
 #endif // Q_OS_MACOS
 
     // Stuff that don't appear in the Options GUI but are saved
