@@ -28,8 +28,6 @@
 
 #pragma once
 
-#include <stdexcept>
-
 #include <QBasicTimer>
 #include <QHash>
 #include <QList>
@@ -38,6 +36,9 @@
 #include <QRegularExpression>
 #include <QSharedPointer>
 
+#include "base/exceptions.h"
+#include "base/settingvalue.h"
+
 class QThread;
 class QTimer;
 
@@ -53,17 +54,16 @@ namespace RSS
 
     class AutoDownloadRule;
 
-    class ParsingError : public std::runtime_error
+    class ParsingError : public RuntimeError
     {
     public:
-        explicit ParsingError(const QString &message);
-        QString message() const;
+        using RuntimeError::RuntimeError;
     };
 
     class AutoDownloader final : public QObject
     {
         Q_OBJECT
-        Q_DISABLE_COPY(AutoDownloader)
+        Q_DISABLE_COPY_MOVE(AutoDownloader)
 
         friend class ::Application;
 
@@ -87,7 +87,7 @@ namespace RSS
         QRegularExpression smartEpisodeRegex() const;
 
         bool downloadRepacks() const;
-        void setDownloadRepacks(bool downloadRepacks);
+        void setDownloadRepacks(bool enabled);
 
         bool hasRule(const QString &ruleName) const;
         AutoDownloadRule ruleByName(const QString &ruleName) const;
@@ -132,7 +132,10 @@ namespace RSS
 
         static QPointer<AutoDownloader> m_instance;
 
-        bool m_processingEnabled;
+        CachedSettingValue<bool> m_storeProcessingEnabled;
+        SettingValue<QVariant> m_storeSmartEpisodeFilter;
+        SettingValue<bool> m_storeDownloadRepacks;
+
         QTimer *m_processingTimer;
         QThread *m_ioThread;
         AsyncFileStorage *m_fileStorage;
