@@ -31,6 +31,7 @@
 #include "rss_feed.h"
 
 #include <algorithm>
+#include <utility>
 #include <vector>
 
 #include <QDir>
@@ -40,14 +41,14 @@
 #include <QJsonValue>
 #include <QUrl>
 
-#include "../asyncfilestorage.h"
-#include "../global.h"
-#include "../logger.h"
-#include "../net/downloadmanager.h"
-#include "../profile.h"
-#include "../utils/fs.h"
-#include "private/rss_parser.h"
+#include "base/asyncfilestorage.h"
+#include "base/global.h"
+#include "base/logger.h"
+#include "base/net/downloadmanager.h"
+#include "base/profile.h"
+#include "base/utils/fs.h"
 #include "rss_article.h"
+#include "rss_parser.h"
 #include "rss_session.h"
 
 const QString KEY_UID(QStringLiteral("uid"));
@@ -66,15 +67,19 @@ Feed::Feed(const QUuid &uid, const QString &url, const QString &path, Session *s
     , m_uid(uid)
     , m_url(url)
 {
-    m_dataFileName = QString::fromLatin1(m_uid.toRfc4122().toHex()) + QLatin1String(".json");
+    const auto uidHex = QString::fromLatin1(m_uid.toRfc4122().toHex());
+    m_dataFileName = uidHex + QLatin1String(".json");
 
     // Move to new file naming scheme (since v4.1.2)
-    const QString legacyFilename {Utils::Fs::toValidFileSystemName(m_url, false, QLatin1String("_"))
+    const QString legacyFilename
+    {Utils::Fs::toValidFileSystemName(m_url, false, QLatin1String("_"))
                 + QLatin1String(".json")};
     const QDir storageDir {m_session->dataFileStorage()->storageDir()};
     if (!QFile::exists(storageDir.absoluteFilePath(m_dataFileName)))
         QFile::rename(storageDir.absoluteFilePath(legacyFilename), storageDir.absoluteFilePath(m_dataFileName));
 
+    m_iconPath = Utils::Fs::toUniformPath(storageDir.absoluteFilePath(uidHex + QLatin1String(".ico")));
+
     m_parser = new Private::Parser(m_lastBuildDate);
     m_parser->moveToThread(m_session->workingThread());
     connect(this, &Feed::destroyed, m_parser, &Private::Parser::deleteLater);
@@ -95,7 +100,6 @@ Feed::Feed(const QUuid &uid, const QString &url, const QString &path, Session *s
 Feed::~Feed()
 {
     emit aboutToBeDestroyed(this);
-    Utils::Fs::forceRemove(m_iconPath);
 }
 
 QList<Article *> Feed::articles() const
@@ -106,8 +110,10 @@ QList<Article *> Feed::articles() const
 void Feed::markAsRead()
 {
     const int oldUnreadCount = m_unreadCount;
-    for (Article *article : asConst(m_articles)) {
-        if (!article->isRead()) {
+    for (Article *article : asConst(m_articles))
+    {
+        if (!article->isRead())
+        {
             article->disconnect(this);
             article->markAsRead();
             --m_unreadCount;
@@ -115,7 +121,8 @@ void Feed::markAsRead()
         }
     }
 
-    if (m_unreadCount != oldUnreadCount) {
+    if (m_unreadCount != oldUnreadCount)
+    {
         m_dirty = true;
         store();
         emit unreadCountChanged(this);
@@ -132,6 +139,9 @@ void Feed::refresh()
     m_downloadHandler = Net::DownloadManager::instance()->download(m_url);
     connect(m_downloadHandler, &Net::DownloadHandler::finished, this, &Feed::handleDownloadFinished);
 
+    if (!QFile::exists(m_iconPath))
+        downloadIcon();
+
     m_isLoading = true;
     emit stateChanged(this);
 }
@@ -180,8 +190,8 @@ void Feed::handleMaxArticlesPerFeedChanged(const int n)
 
 void Feed::handleIconDownloadFinished(const Net::DownloadResult &result)
 {
-    if (result.status == Net::DownloadStatus::Success) {
-        m_iconPath = Utils::Fs::toUniformPath(result.filePath);
+    if (result.status == Net::DownloadStatus::Success)
+    {
         emit iconLoaded(this);
     }
 }
@@ -195,13 +205,15 @@ void Feed::handleDownloadFinished(const Net::DownloadResult &result)
 {
     m_downloadHandler = nullptr; // will be deleted by DownloadManager later
 
-    if (result.status == Net::DownloadStatus::Success) {
+    if (result.status == Net::DownloadStatus::Success)
+    {
         LogMsg(tr("RSS feed at '%1' is successfully downloaded. Starting to parse it.")
                 .arg(result.url));
         // Parse the download RSS
         m_parser->parse(result.data);
     }
-    else {
+    else
+    {
         m_isLoading = false;
         m_hasError = true;
 
@@ -216,13 +228,15 @@ void Feed::handleParsingFinished(const RSS::Private::ParsingResult &result)
 {
     m_hasError = !result.error.isEmpty();
 
-    if (!result.title.isEmpty() && (title() != result.title)) {
+    if (!result.title.isEmpty() && (title() != result.title))
+    {
         m_title = result.title;
         m_dirty = true;
         emit titleChanged(this);
     }
 
-    if (!result.lastBuildDate.isEmpty()) {
+    if (!result.lastBuildDate.isEmpty())
+    {
         m_lastBuildDate = result.lastBuildDate;
         m_dirty = true;
     }
@@ -234,7 +248,8 @@ void Feed::handleParsingFinished(const RSS::Private::ParsingResult &result)
     const int newArticlesCount = updateArticles(result.articles);
     store();
 
-    if (m_hasError) {
+    if (m_hasError)
+    {
         LogMsg(tr("Failed to parse RSS feed at '%1'. Reason: %2").arg(m_url, result.error)
                , Log::WARNING);
     }
@@ -249,16 +264,19 @@ void Feed::load()
 {
     QFile file(m_session->dataFileStorage()->storageDir().absoluteFilePath(m_dataFileName));
 
-    if (!file.exists()) {
+    if (!file.exists())
+    {
         loadArticlesLegacy();
         m_dirty = true;
         store(); // convert to new format
     }
-    else if (file.open(QFile::ReadOnly)) {
+    else if (file.open(QFile::ReadOnly))
+    {
         loadArticles(file.readAll());
         file.close();
     }
-    else {
+    else
+    {
         LogMsg(tr("Couldn't read RSS Session data from %1. Error: %2")
                .arg(m_dataFileName, file.errorString())
                , Log::WARNING);
@@ -269,33 +287,38 @@ void Feed::loadArticles(const QByteArray &data)
 {
     QJsonParseError jsonError;
     const QJsonDocument jsonDoc = QJsonDocument::fromJson(data, &jsonError);
-    if (jsonError.error != QJsonParseError::NoError) {
+    if (jsonError.error != QJsonParseError::NoError)
+    {
         LogMsg(tr("Couldn't parse RSS Session data. Error: %1").arg(jsonError.errorString())
                , Log::WARNING);
         return;
     }
 
-    if (!jsonDoc.isArray()) {
+    if (!jsonDoc.isArray())
+    {
         LogMsg(tr("Couldn't load RSS Session data. Invalid data format."), Log::WARNING);
         return;
     }
 
     const QJsonArray jsonArr = jsonDoc.array();
     int i = -1;
-    for (const QJsonValue &jsonVal : jsonArr) {
+    for (const QJsonValue &jsonVal : jsonArr)
+    {
         ++i;
-        if (!jsonVal.isObject()) {
+        if (!jsonVal.isObject())
+        {
             LogMsg(tr("Couldn't load RSS article '%1#%2'. Invalid data format.").arg(m_url).arg(i)
                    , Log::WARNING);
             continue;
         }
 
-        try {
+        try
+        {
             auto article = new Article(this, jsonVal.toObject());
             if (!addArticle(article))
                 delete article;
         }
-        catch (const std::runtime_error&) {}
+        catch (const RuntimeError &) {}
     }
 }
 
@@ -304,18 +327,20 @@ void Feed::loadArticlesLegacy()
     const SettingsPtr qBTRSSFeeds = Profile::instance()->applicationSettings(QStringLiteral("qBittorrent-rss-feeds"));
     const QVariantHash allOldItems = qBTRSSFeeds->value("old_items").toHash();
 
-    for (const QVariant &var : asConst(allOldItems.value(m_url).toList())) {
+    for (const QVariant &var : asConst(allOldItems.value(m_url).toList()))
+    {
         auto hash = var.toHash();
         // update legacy keys
         hash[Article::KeyLink] = hash.take(QLatin1String("news_link"));
         hash[Article::KeyTorrentURL] = hash.take(QLatin1String("torrent_url"));
         hash[Article::KeyIsRead] = hash.take(QLatin1String("read"));
-        try {
+        try
+        {
             auto article = new Article(this, hash);
             if (!addArticle(article))
                 delete article;
         }
-        catch (const std::runtime_error&) {}
+        catch (const RuntimeError &) {}
     }
 }
 
@@ -353,7 +378,8 @@ bool Feed::addArticle(Article *article)
 
     m_articles[article->guid()] = article;
     m_articlesByDate.insert(lowerBound, article);
-    if (!article->isRead()) {
+    if (!article->isRead())
+    {
         increaseUnreadCount();
         connect(article, &Article::read, this, &Feed::handleArticleRead);
     }
@@ -402,7 +428,7 @@ void Feed::downloadIcon()
     const QUrl url(m_url);
     const auto iconUrl = QString::fromLatin1("%1://%2/favicon.ico").arg(url.scheme(), url.host());
     Net::DownloadManager::instance()->download(
-            Net::DownloadRequest(iconUrl).saveToFile(true)
+            Net::DownloadRequest(iconUrl).saveToFile(true).destFileName(m_iconPath)
                 , this, &Feed::handleIconDownloadFinished);
 }
 
@@ -414,12 +440,14 @@ int Feed::updateArticles(const QList<QVariantHash> &loadedArticles)
     QDateTime dummyPubDate {QDateTime::currentDateTime()};
     QVector<QVariantHash> newArticles;
     newArticles.reserve(loadedArticles.size());
-    for (QVariantHash article : loadedArticles) {
+    for (QVariantHash article : loadedArticles)
+    {
         // If article has no publication date we use feed update time as a fallback.
         // To prevent processing of "out-of-limit" articles we must not assign dates
         // that are earlier than the dates of existing articles.
         const Article *existingArticle = articleByGUID(article[Article::KeyId].toString());
-        if (existingArticle) {
+        if (existingArticle)
+        {
             dummyPubDate = existingArticle->date().addMSecs(-1);
             continue;
         }
@@ -434,19 +462,19 @@ int Feed::updateArticles(const QList<QVariantHash> &loadedArticles)
     if (newArticles.empty())
         return 0;
 
-    using ArticleSortAdaptor = QPair<QDateTime, const QVariantHash *>;
+    using ArticleSortAdaptor = std::pair<QDateTime, const QVariantHash *>;
     std::vector<ArticleSortAdaptor> sortData;
     const QList<Article *> existingArticles = articles();
     sortData.reserve(existingArticles.size() + newArticles.size());
     std::transform(existingArticles.begin(), existingArticles.end(), std::back_inserter(sortData)
                    , [](const Article *article)
     {
-        return qMakePair(article->date(), nullptr);
+        return std::make_pair(article->date(), nullptr);
     });
     std::transform(newArticles.begin(), newArticles.end(), std::back_inserter(sortData)
                    , [](const QVariantHash &article)
     {
-        return qMakePair(article[Article::KeyDate].toDateTime(), &article);
+        return std::make_pair(article[Article::KeyDate].toDateTime(), &article);
     });
 
     // Sort article list in reverse chronological order
@@ -462,7 +490,8 @@ int Feed::updateArticles(const QList<QVariantHash> &loadedArticles)
     int newArticlesCount = 0;
     std::for_each(sortData.crbegin(), sortData.crend(), [this, &newArticlesCount](const ArticleSortAdaptor &a)
     {
-        if (a.second) {
+        if (a.second)
+        {
             addArticle(new Article {this, *a.second});
             ++newArticlesCount;
         }
@@ -482,7 +511,8 @@ QJsonValue Feed::toJsonValue(const bool withData) const
     jsonObj.insert(KEY_UID, uid().toString());
     jsonObj.insert(KEY_URL, url());
 
-    if (withData) {
+    if (withData)
+    {
         jsonObj.insert(KEY_TITLE, title());
         jsonObj.insert(KEY_LASTBUILDDATE, lastBuildDate());
         jsonObj.insert(KEY_ISLOADING, isLoading());
@@ -499,7 +529,8 @@ QJsonValue Feed::toJsonValue(const bool withData) const
 
 void Feed::handleSessionProcessingEnabledChanged(const bool enabled)
 {
-    if (enabled) {
+    if (enabled)
+    {
         downloadIcon();
         disconnect(m_session, &Session::processingStateChanged
                    , this, &Feed::handleSessionProcessingEnabledChanged);
@@ -519,6 +550,7 @@ void Feed::handleArticleRead(Article *article)
 void Feed::cleanup()
 {
     Utils::Fs::forceRemove(m_session->dataFileStorage()->storageDir().absoluteFilePath(m_dataFileName));
+    Utils::Fs::forceRemove(m_iconPath);
 }
 
 void Feed::timerEvent(QTimerEvent *event)
