@@ -73,6 +73,9 @@
 #include <QPointer>
 #include <QTimer>
 
+#include "base/3rdparty/expected.hpp"
+#include "base/settingvalue.h"
+
 class QThread;
 
 class Application;
@@ -87,7 +90,7 @@ namespace RSS
     class Session : public QObject
     {
         Q_OBJECT
-        Q_DISABLE_COPY(Session)
+        Q_DISABLE_COPY_MOVE(Session)
 
         friend class ::Application;
 
@@ -110,12 +113,11 @@ namespace RSS
         int refreshInterval() const;
         void setRefreshInterval(int refreshInterval);
 
-        bool addFolder(const QString &path, QString *error = nullptr);
-        bool addFeed(const QString &url, const QString &path, QString *error = nullptr);
-        bool moveItem(const QString &itemPath, const QString &destPath
-                         , QString *error = nullptr);
-        bool moveItem(Item *item, const QString &destPath, QString *error = nullptr);
-        bool removeItem(const QString &itemPath, QString *error = nullptr);
+        nonstd::expected<void, QString> addFolder(const QString &path);
+        nonstd::expected<void, QString> addFeed(const QString &url, const QString &path);
+        nonstd::expected<void, QString> moveItem(const QString &itemPath, const QString &destPath);
+        nonstd::expected<void, QString> moveItem(Item *item, const QString &destPath);
+        nonstd::expected<void, QString> removeItem(const QString &itemPath);
 
         QList<Item *> items() const;
         Item *itemByPath(const QString &path) const;
@@ -146,20 +148,20 @@ namespace RSS
         void loadFolder(const QJsonObject &jsonObj, Folder *folder);
         void loadLegacy();
         void store();
-        Folder *prepareItemDest(const QString &path, QString *error);
+        nonstd::expected<Folder *, QString> prepareItemDest(const QString &path);
         Folder *addSubfolder(const QString &name, Folder *parentFolder);
         Feed *addFeedToFolder(const QUuid &uid, const QString &url, const QString &name, Folder *parentFolder);
         void addItem(Item *item, Folder *destFolder);
 
         static QPointer<Session> m_instance;
 
-        bool m_processingEnabled;
+        CachedSettingValue<bool> m_storeProcessingEnabled;
+        CachedSettingValue<int> m_storeRefreshInterval;
+        CachedSettingValue<int> m_storeMaxArticlesPerFeed;
         QThread *m_workingThread;
         AsyncFileStorage *m_confFileStorage;
         AsyncFileStorage *m_dataFileStorage;
         QTimer m_refreshTimer;
-        int m_refreshInterval;
-        int m_maxArticlesPerFeed;
         QHash<QString, Item *> m_itemsByPath;
         QHash<QUuid, Feed *> m_feedsByUID;
         QHash<QString, Feed *> m_feedsByURL;
