@@ -35,6 +35,10 @@
 #include <QSystemTrayIcon>
 #endif
 
+#include "base/bittorrent/torrent.h"
+#include "base/logger.h"
+#include "base/settingvalue.h"
+
 class QCloseEvent;
 class QFileSystemWatcher;
 class QSplitter;
@@ -47,6 +51,7 @@ class ExecutionLogWidget;
 class LineEdit;
 class OptionsDialog;
 class PowerManagement;
+class ProgramUpdater;
 class PropertiesWidget;
 class RSSWidget;
 class SearchWidget;
@@ -56,11 +61,6 @@ class TorrentCreatorDialog;
 class TransferListFiltersWidget;
 class TransferListWidget;
 
-namespace BitTorrent
-{
-    class Torrent;
-}
-
 namespace Net
 {
     struct DownloadResult;
@@ -82,19 +82,22 @@ public:
     QWidget *currentTabWidget() const;
     TransferListWidget *transferListWidget() const;
     PropertiesWidget *propertiesWidget() const;
-    QMenu *trayIconMenu();
 
     // ExecutionLog properties
     bool isExecutionLogEnabled() const;
     void setExecutionLogEnabled(bool value);
-    int executionLogMsgTypes() const;
-    void setExecutionLogMsgTypes(int value);
+    Log::MsgTypes executionLogMsgTypes() const;
+    void setExecutionLogMsgTypes(Log::MsgTypes value);
 
     // Notifications properties
     bool isNotificationsEnabled() const;
     void setNotificationsEnabled(bool value);
     bool isTorrentAddedNotificationsEnabled() const;
     void setTorrentAddedNotificationsEnabled(bool value);
+#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
+    int getNotificationTimeout() const;
+    void setNotificationTimeout(int value);
+#endif
 
     // Misc properties
     bool isDownloadTrackerFavicon() const;
@@ -103,7 +106,10 @@ public:
     void activate();
     void cleanup();
 
-    void showNotificationBaloon(const QString &title, const QString &msg) const;
+    void showNotificationBalloon(const QString &title, const QString &msg) const;
+
+signals:
+    void systemTrayIconCreated();
 
 private slots:
     void showFilterContextMenu(const QPoint &);
@@ -128,15 +134,12 @@ private slots:
     void focusSearchFilter();
     void reloadSessionStats();
     void reloadTorrentStats(const QVector<BitTorrent::Torrent *> &torrents);
-    void loadPreferences(bool configureSession = true);
+    void loadPreferences();
     void addTorrentFailed(const QString &error) const;
     void torrentNew(BitTorrent::Torrent *const torrent) const;
     void finishedTorrent(BitTorrent::Torrent *const torrent) const;
     void askRecursiveTorrentDownloadConfirmation(BitTorrent::Torrent *const torrent);
     void optionsSaved();
-#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
-    void handleUpdateCheckFinished(bool updateAvailable, QString newVersion, bool invokedByUser);
-#endif
     void toggleAlternativeSpeeds();
 
 #ifdef Q_OS_WIN
@@ -177,9 +180,7 @@ private slots:
     void on_actionLock_triggered();
     // Check for unpaused downloading or seeding torrents and prevent system suspend/sleep according to preferences
     void updatePowerManagementState();
-#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
-    void checkProgramUpdate();
-#endif
+
     void toolbarMenuRequested(const QPoint &point);
     void toolbarIconsOnly();
     void toolbarTextOnly();
@@ -190,16 +191,14 @@ private slots:
     void on_actionCloseWindow_triggered();
 #else
     void toggleVisibility(const QSystemTrayIcon::ActivationReason reason = QSystemTrayIcon::Trigger);
-    void createSystrayDelayed();
-    void updateTrayIconMenu();
 #endif
 
 private:
+    void createTrayIconMenu();
 #ifdef Q_OS_MACOS
     void setupDockClickHandler();
 #else
-    void createTrayIcon();
-    QIcon getSystrayIcon() const;
+    void createTrayIcon(int retries);
 #endif
 #ifdef Q_OS_WIN
     void installPython();
@@ -220,7 +219,7 @@ private:
 
     QFileSystemWatcher *m_executableWatcher;
     // GUI related
-    bool m_posInitialized;
+    bool m_posInitialized = false;
     QPointer<QTabWidget> m_tabs;
     QPointer<StatusBar> m_statusBar;
     QPointer<OptionsDialog> m_options;
@@ -228,18 +227,19 @@ private:
     QPointer<StatsDialog> m_statsDlg;
     QPointer<TorrentCreatorDialog> m_createTorrentDlg;
     QPointer<DownloadFromURLDialog> m_downloadFromURLDialog;
+
 #ifndef Q_OS_MACOS
     QPointer<QSystemTrayIcon> m_systrayIcon;
-    QPointer<QTimer> m_systrayCreator;
 #endif
     QPointer<QMenu> m_trayIconMenu;
+
     TransferListWidget *m_transferListWidget;
     TransferListFiltersWidget *m_transferListFiltersWidget;
     PropertiesWidget *m_propertiesWidget;
     bool m_displaySpeedInTitle;
-    bool m_forceExit;
+    bool m_forceExit = false;
     bool m_uiLocked;
-    bool m_unlockDlgShowing;
+    bool m_unlockDlgShowing = false;
     LineEdit *m_searchFilter;
     QAction *m_searchFilterAction;
     // Widgets
@@ -252,10 +252,23 @@ private:
     // Power Management
     PowerManagement *m_pwr;
     QTimer *m_preventTimer;
-#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
-    QTimer *m_programUpdateTimer;
-    bool m_wasUpdateCheckEnabled;
-#endif
-    bool m_hasPython;
+    bool m_hasPython = false;
     QMenu *m_toolbarMenu;
+
+    SettingValue<bool> m_storeExecutionLogEnabled;
+    SettingValue<bool> m_storeDownloadTrackerFavicon;
+    SettingValue<bool> m_storeNotificationEnabled;
+    SettingValue<bool> m_storeNotificationTorrentAdded;
+    CachedSettingValue<Log::MsgTypes> m_storeExecutionLogTypes;
+
+#if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)) && defined(QT_DBUS_LIB)
+    SettingValue<int> m_storeNotificationTimeOut;
+#endif
+
+#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
+    void checkProgramUpdate(bool invokedByUser);
+    void handleUpdateCheckFinished(ProgramUpdater *updater, bool invokedByUser);
+
+    QTimer *m_programUpdateTimer = nullptr;
+#endif
 };
