@@ -43,6 +43,7 @@
 #include <QTranslator>
 
 #include "base/bittorrent/session.h"
+#include "base/exceptions.h"
 #include "base/global.h"
 #include "base/net/dnsupdater.h"
 #include "base/net/portforwarder.h"
@@ -50,8 +51,8 @@
 #include "base/preferences.h"
 #include "base/rss/rss_autodownloader.h"
 #include "base/rss/rss_session.h"
-#include "base/scanfoldersmodel.h"
 #include "base/torrentfileguard.h"
+#include "base/torrentfileswatcher.h"
 #include "base/unicodestrings.h"
 #include "base/utils/fs.h"
 #include "base/utils/net.h"
@@ -63,10 +64,11 @@
 #include "banlistoptionsdialog.h"
 #include "ipsubnetwhitelistoptionsdialog.h"
 #include "rss/automatedrssdownloader.h"
-#include "scanfoldersdelegate.h"
 #include "ui_optionsdialog.h"
 #include "uithememanager.h"
 #include "utils.h"
+#include "watchedfolderoptionsdialog.h"
+#include "watchedfoldersmodel.h"
 
 #define SETTINGS_KEY(name) "OptionsDialog/" name
 
@@ -131,8 +133,10 @@ namespace
         case QLocale::Latvian: return QString::fromUtf8(C_LOCALE_LATVIAN);
         case QLocale::Lithuanian: return QString::fromUtf8(C_LOCALE_LITHUANIAN);
         case QLocale::Malay: return QString::fromUtf8(C_LOCALE_MALAY);
+        case QLocale::Mongolian: return QString::fromUtf8(C_LOCALE_MONGOLIAN);
         case QLocale::NorwegianBokmal: return QString::fromUtf8(C_LOCALE_NORWEGIAN);
         case QLocale::Occitan: return QString::fromUtf8(C_LOCALE_OCCITAN);
+        case QLocale::Persian: return QString::fromUtf8(C_LOCALE_PERSIAN);
         case QLocale::Polish: return QString::fromUtf8(C_LOCALE_POLISH);
         case QLocale::Portuguese:
             if (locale.country() == QLocale::Brazil)
@@ -145,6 +149,7 @@ namespace
         case QLocale::Slovenian: return QString::fromUtf8(C_LOCALE_SLOVENIAN);
         case QLocale::Spanish: return QString::fromUtf8(C_LOCALE_SPANISH);
         case QLocale::Swedish: return QString::fromUtf8(C_LOCALE_SWEDISH);
+        case QLocale::Thai: return QString::fromUtf8(C_LOCALE_THAI);
         case QLocale::Turkish: return QString::fromUtf8(C_LOCALE_TURKISH);
         case QLocale::Ukrainian: return QString::fromUtf8(C_LOCALE_UKRAINIAN);
         case QLocale::Uzbek: return QString::fromUtf8(C_LOCALE_UZBEK);
@@ -175,6 +180,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     , m_ui {new Ui::OptionsDialog}
     , m_storeDialogSize {SETTINGS_KEY("Size")}
     , m_storeHSplitterSize {SETTINGS_KEY("HorizontalSplitterSizes")}
+    , m_storeLastViewedPage {SETTINGS_KEY("LastViewedPage")}
 {
     qDebug("-> Constructing Options");
     m_ui->setupUi(this);
@@ -211,8 +217,8 @@ OptionsDialog::OptionsDialog(QWidget *parent)
 
     m_ui->IpFilterRefreshBtn->setIcon(UIThemeManager::instance()->getIcon("view-refresh"));
 
-    m_ui->labelGlobalRate->setPixmap(Utils::Gui::scaledPixmap(UIThemeManager::instance()->getIcon(QLatin1String("slow_off")), this, 16));
-    m_ui->labelAltRate->setPixmap(Utils::Gui::scaledPixmap(UIThemeManager::instance()->getIcon(QLatin1String("slow")), this, 16));
+    m_ui->labelGlobalRate->setPixmap(Utils::Gui::scaledPixmap(UIThemeManager::instance()->getIcon(QLatin1String("slow_off")), this, 24));
+    m_ui->labelAltRate->setPixmap(Utils::Gui::scaledPixmap(UIThemeManager::instance()->getIcon(QLatin1String("slow")), this, 24));
 
     m_ui->deleteTorrentWarningIcon->setPixmap(QApplication::style()->standardIcon(QStyle::SP_MessageBoxCritical).pixmap(16, 16));
     m_ui->deleteTorrentWarningIcon->hide();
@@ -233,23 +239,16 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     m_ui->hsplitter->setCollapsible(0, false);
     m_ui->hsplitter->setCollapsible(1, false);
     // Get apply button in button box
-    const QList<QAbstractButton *> buttons = m_ui->buttonBox->buttons();
-    for (QAbstractButton *button : buttons)
-    {
-        if (m_ui->buttonBox->buttonRole(button) == QDialogButtonBox::ApplyRole)
-        {
-            m_applyButton = button;
-            break;
-        }
-    }
+    m_applyButton = m_ui->buttonBox->button(QDialogButtonBox::Apply);
+    connect(m_applyButton, &QPushButton::clicked, this, &OptionsDialog::applySettings);
 
+    auto watchedFoldersModel = new WatchedFoldersModel(TorrentFilesWatcher::instance(), this);
+    connect(watchedFoldersModel, &QAbstractListModel::dataChanged, this, &ThisType::enableApplyButton);
     m_ui->scanFoldersView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
-    m_ui->scanFoldersView->setModel(ScanFoldersModel::instance());
-    m_ui->scanFoldersView->setItemDelegate(new ScanFoldersDelegate(this, m_ui->scanFoldersView));
-    connect(ScanFoldersModel::instance(), &QAbstractListModel::dataChanged, this, &ThisType::enableApplyButton);
-    connect(m_ui->scanFoldersView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &ThisType::handleScanFolderViewSelectionChanged);
+    m_ui->scanFoldersView->setModel(watchedFoldersModel);
+    connect(m_ui->scanFoldersView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &ThisType::handleWatchedFolderViewSelectionChanged);
+    connect(m_ui->scanFoldersView, &QTreeView::doubleClicked, this, &ThisType::editWatchedFolderOptions);
 
-    connect(m_ui->buttonBox, &QDialogButtonBox::clicked, this, &OptionsDialog::applySettings);
     // Languages supported
     initializeLanguageCombo();
 
@@ -257,7 +256,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     m_ui->customThemeFilePath->setSelectedPath(Preferences::instance()->customUIThemePath());
     m_ui->customThemeFilePath->setMode(FileSystemPathEdit::Mode::FileOpen);
     m_ui->customThemeFilePath->setDialogCaption(tr("Select qBittorrent UI Theme file"));
-    m_ui->customThemeFilePath->setFileNameFilter(tr("qBittorrent UI Theme file (*.qbtheme)"));
+    m_ui->customThemeFilePath->setFileNameFilter(tr("qBittorrent UI Theme file (*.qbtheme config.json)"));
 
 #if (defined(Q_OS_UNIX) && !defined(Q_OS_MACOS))
     m_ui->checkUseSystemIcon->setChecked(Preferences::instance()->useSystemIconTheme());
@@ -302,7 +301,6 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     void (QSpinBox::*qSpinBoxValueChanged)(int) = &QSpinBox::valueChanged;
 
     connect(m_ui->comboProxyType, qComboBoxCurrentIndexChanged, this, &ThisType::enableProxy);
-    connect(m_ui->checkRandomPort, &QAbstractButton::toggled, m_ui->spinPort, &ThisType::setDisabled);
 
     // Apply button is activated when a value is changed
     // Behavior tab
@@ -356,7 +354,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->comboTorrentCategoryChanged, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->comboCategoryDefaultPathChanged, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->comboCategoryChanged, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
-    connect(m_ui->textTempPath, &FileSystemPathEdit::selectedPathChanged, this, &ThisType::enableApplyButton);
+    connect(m_ui->textDownloadPath, &FileSystemPathEdit::selectedPathChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->checkAppendqB, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkPreallocateAll, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkRecursiveDownload, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
@@ -374,10 +372,10 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->textExportDirFin, &FileSystemPathEdit::selectedPathChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->actionTorrentDlOnDblClBox, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->actionTorrentFnOnDblClBox, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
-    connect(m_ui->checkTempFolder, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
-    connect(m_ui->checkTempFolder, &QAbstractButton::toggled, m_ui->textTempPath, &QWidget::setEnabled);
-    connect(m_ui->addScanFolderButton, &QAbstractButton::clicked, this, &ThisType::enableApplyButton);
-    connect(m_ui->removeScanFolderButton, &QAbstractButton::clicked, this, &ThisType::enableApplyButton);
+    connect(m_ui->checkUseDownloadPath, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
+    connect(m_ui->checkUseDownloadPath, &QAbstractButton::toggled, m_ui->textDownloadPath, &QWidget::setEnabled);
+    connect(m_ui->addWatchedFolderButton, &QAbstractButton::clicked, this, &ThisType::enableApplyButton);
+    connect(m_ui->removeWatchedFolderButton, &QAbstractButton::clicked, this, &ThisType::enableApplyButton);
     connect(m_ui->groupMailNotification, &QGroupBox::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->senderEmailTxt, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->lineEditDestEmail, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
@@ -390,7 +388,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->lineEditAutoRun, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->autoRunConsole, &QCheckBox::toggled, this, &ThisType::enableApplyButton);
 
-    const QString autoRunStr = QString("%1\n    %2\n    %3\n    %4\n    %5\n    %6\n    %7\n    %8\n    %9\n    %10\n    %11\n%12")
+    const QString autoRunStr = QString("%1\n    %2\n    %3\n    %4\n    %5\n    %6\n    %7\n    %8\n    %9\n    %10\n    %11\n    %12\n    %13\n%14")
         .arg(tr("Supported parameters (case sensitive):")
             , tr("%N: Torrent name")
             , tr("%L: Category")
@@ -401,14 +399,15 @@ OptionsDialog::OptionsDialog(QWidget *parent)
             , tr("%C: Number of files")
             , tr("%Z: Torrent size (bytes)"))
         .arg(tr("%T: Current tracker")
-            , tr("%I: Info hash")
+            , tr("%I: Info hash v1 (or '-' if unavailable)")
+            , tr("%J: Info hash v2 (or '-' if unavailable)")
+            , tr("%K: Torrent ID (either sha-1 info hash for v1 torrent or truncated sha-256 info hash for v2/hybrid torrent)")
             , tr("Tip: Encapsulate parameter with quotation marks to avoid text being cut off at whitespace (e.g., \"%N\")"));
     m_ui->labelAutoRunParam->setText(autoRunStr);
 
     // Connection tab
     connect(m_ui->comboProtocol, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->spinPort, qSpinBoxValueChanged, this, &ThisType::enableApplyButton);
-    connect(m_ui->checkRandomPort, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkUPnP, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->spinUploadLimit, qSpinBoxValueChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->spinDownloadLimit, qSpinBoxValueChanged, this, &ThisType::enableApplyButton);
@@ -422,8 +421,8 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->checkLimitTransportOverhead, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkLimitLocalPeerRate, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     // Bittorrent tab
-    connect(m_ui->checkMaxConnecs, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
-    connect(m_ui->checkMaxConnecsPerTorrent, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
+    connect(m_ui->checkMaxConnections, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
+    connect(m_ui->checkMaxConnectionsPerTorrent, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkMaxUploads, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkMaxUploadsPerTorrent, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->spinMaxConnec, qSpinBoxValueChanged, this, &ThisType::enableApplyButton);
@@ -447,7 +446,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->comboProxyType, qComboBoxCurrentIndexChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->textProxyIP, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->spinProxyPort, qSpinBoxValueChanged, this, &ThisType::enableApplyButton);
-    connect(m_ui->checkProxyPeerConnecs, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
+    connect(m_ui->checkProxyPeerConnections, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->isProxyOnlyForTorrents, &QAbstractButton::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->checkProxyAuth, &QGroupBox::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->textProxyUsername, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
@@ -517,6 +516,8 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     connect(m_ui->textWebUIRootFolder, &FileSystemPathLineEdit::selectedPathChanged, this, &ThisType::enableApplyButton);
     connect(m_ui->groupWebUIAddCustomHTTPHeaders, &QGroupBox::toggled, this, &ThisType::enableApplyButton);
     connect(m_ui->textWebUICustomHTTPHeaders, &QPlainTextEdit::textChanged, this, &OptionsDialog::enableApplyButton);
+    connect(m_ui->groupEnableReverseProxySupport, &QGroupBox::toggled, this, &ThisType::enableApplyButton);
+    connect(m_ui->textTrustedReverseProxiesList, &QLineEdit::textChanged, this, &ThisType::enableApplyButton);
 #endif // DISABLE_WEBUI
 
     // RSS tab
@@ -558,8 +559,8 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     m_ui->textSavePath->setDialogCaption(tr("Choose a save directory"));
     m_ui->textSavePath->setMode(FileSystemPathEdit::Mode::DirectorySave);
 
-    m_ui->textTempPath->setDialogCaption(tr("Choose a save directory"));
-    m_ui->textTempPath->setMode(FileSystemPathEdit::Mode::DirectorySave);
+    m_ui->textDownloadPath->setDialogCaption(tr("Choose a save directory"));
+    m_ui->textDownloadPath->setMode(FileSystemPathEdit::Mode::DirectorySave);
 
     // disable mouse wheel event on widgets to avoid mis-selection
     auto *wheelEventEater = new WheelEventEater(this);
@@ -568,6 +569,8 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     for (QSpinBox *widget : asConst(findChildren<QSpinBox *>()))
         widget->installEventFilter(wheelEventEater);
 
+    m_ui->tabSelection->setCurrentRow(m_storeLastViewedPage);
+
     Utils::Gui::resize(this, m_storeDialogSize);
     show();
     // Have to be called after show(), because splitter width needed
@@ -617,9 +620,8 @@ OptionsDialog::~OptionsDialog()
         hSplitterSizes.append(QString::number(size));
     m_storeHSplitterSize = hSplitterSizes;
 
-    for (const QString &path : asConst(m_addedScanDirs))
-        ScanFoldersModel::instance()->removePath(path);
-    ScanFoldersModel::instance()->configure(); // reloads "removed" paths
+    m_storeLastViewedPage = m_ui->tabSelection->currentRow();
+
     delete m_ui;
 }
 
@@ -675,7 +677,7 @@ void OptionsDialog::saveOptions()
     pref->setHideZeroValues(m_ui->checkHideZero->isChecked());
     pref->setHideZeroComboValues(m_ui->comboHideZero->currentIndex());
 #ifndef Q_OS_MACOS
-    pref->setSystrayIntegration(systrayIntegration());
+    pref->setSystemTrayEnabled(systemTrayEnabled());
     pref->setTrayIconStyle(TrayIcon::Style(m_ui->comboTrayIcon->currentIndex()));
     pref->setCloseToTray(closeToTray());
     pref->setMinimizeToTray(minimizeToTray());
@@ -723,20 +725,20 @@ void OptionsDialog::saveOptions()
     RSS::Session::instance()->setMaxArticlesPerFeed(m_ui->spinRSSMaxArticlesPerFeed->value());
     RSS::Session::instance()->setProcessingEnabled(m_ui->checkRSSEnable->isChecked());
     RSS::AutoDownloader::instance()->setProcessingEnabled(m_ui->checkRSSAutoDownloaderEnable->isChecked());
-    RSS::AutoDownloader::instance()->setSmartEpisodeFilters(m_ui->textSmartEpisodeFilters->toPlainText().split('\n', QString::SplitBehavior::SkipEmptyParts));
+    RSS::AutoDownloader::instance()->setSmartEpisodeFilters(m_ui->textSmartEpisodeFilters->toPlainText().split('\n', Qt::SkipEmptyParts));
     RSS::AutoDownloader::instance()->setDownloadRepacks(m_ui->checkSmartFilterDownloadRepacks->isChecked());
 
     auto session = BitTorrent::Session::instance();
 
     // Downloads preferences
-    session->setDefaultSavePath(Utils::Fs::expandPathAbs(m_ui->textSavePath->selectedPath()));
+    session->setSavePath(Utils::Fs::expandPathAbs(m_ui->textSavePath->selectedPath()));
     session->setSubcategoriesEnabled(m_ui->checkUseSubcategories->isChecked());
     session->setAutoTMMDisabledByDefault(m_ui->comboSavingMode->currentIndex() == 0);
     session->setDisableAutoTMMWhenCategoryChanged(m_ui->comboTorrentCategoryChanged->currentIndex() == 1);
     session->setDisableAutoTMMWhenCategorySavePathChanged(m_ui->comboCategoryChanged->currentIndex() == 1);
     session->setDisableAutoTMMWhenDefaultSavePathChanged(m_ui->comboCategoryDefaultPathChanged->currentIndex() == 1);
-    session->setTempPathEnabled(m_ui->checkTempFolder->isChecked());
-    session->setTempPath(Utils::Fs::expandPathAbs(m_ui->textTempPath->selectedPath()));
+    session->setDownloadPathEnabled(m_ui->checkUseDownloadPath->isChecked());
+    session->setDownloadPath(Utils::Fs::expandPathAbs(m_ui->textDownloadPath->selectedPath()));
     session->setAppendExtensionEnabled(m_ui->checkAppendqB->isChecked());
     session->setPreallocationEnabled(preAllocateAllFiles());
     pref->disableRecursiveDownload(!m_ui->checkRecursiveDownload->isChecked());
@@ -744,11 +746,8 @@ void OptionsDialog::saveOptions()
     AddNewTorrentDialog::setTopLevel(m_ui->checkAdditionDialogFront->isChecked());
     session->setAddTorrentPaused(addTorrentsInPause());
     session->setTorrentContentLayout(static_cast<BitTorrent::TorrentContentLayout>(m_ui->contentLayoutComboBox->currentIndex()));
-    ScanFoldersModel::instance()->removeFromFSWatcher(m_removedScanDirs);
-    ScanFoldersModel::instance()->addToFSWatcher(m_addedScanDirs);
-    ScanFoldersModel::instance()->makePersistent();
-    m_removedScanDirs.clear();
-    m_addedScanDirs.clear();
+    auto watchedFoldersModel = static_cast<WatchedFoldersModel *>(m_ui->scanFoldersView->model());
+    watchedFoldersModel->apply();
     session->setTorrentExportDirectory(getTorrentExportDir());
     session->setFinishedTorrentExportDirectory(getFinishedTorrentExportDir());
     pref->setMailNotificationEnabled(m_ui->groupMailNotification->isChecked());
@@ -761,11 +760,11 @@ void OptionsDialog::saveOptions()
     pref->setMailNotificationSMTPPassword(m_ui->mailNotifPassword->text());
     pref->setAutoRunEnabled(m_ui->autoRunBox->isChecked());
     pref->setAutoRunProgram(m_ui->lineEditAutoRun->text().trimmed());
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)) && defined(Q_OS_WIN)
+#if defined(Q_OS_WIN)
     pref->setAutoRunConsoleEnabled(m_ui->autoRunConsole->isChecked());
 #endif
-    pref->setActionOnDblClOnTorrentDl(getActionOnDblClOnTorrentDl());
-    pref->setActionOnDblClOnTorrentFn(getActionOnDblClOnTorrentFn());
+    pref->setActionOnDblClOnTorrentDl(m_ui->actionTorrentDlOnDblClBox->currentData().toInt());
+    pref->setActionOnDblClOnTorrentFn(m_ui->actionTorrentFnOnDblClBox->currentData().toInt());
     TorrentFileGuard::setAutoDeleteMode(!m_ui->deleteTorrentBox->isChecked() ? TorrentFileGuard::Never
                              : !m_ui->deleteCancelledTorrentBox->isChecked() ? TorrentFileGuard::IfAdded
                              : TorrentFileGuard::Always);
@@ -774,7 +773,6 @@ void OptionsDialog::saveOptions()
     // Connection preferences
     session->setBTProtocol(static_cast<BitTorrent::BTProtocol>(m_ui->comboProtocol->currentIndex()));
     session->setPort(getPort());
-    session->setUseRandomPort(m_ui->checkRandomPort->isChecked());
     Net::PortForwarder::instance()->setEnabled(isUPnPEnabled());
     session->setGlobalDownloadSpeedLimit(m_ui->spinDownloadLimit->value() * 1024);
     session->setGlobalUploadSpeedLimit(m_ui->spinUploadLimit->value() * 1024);
@@ -785,7 +783,7 @@ void OptionsDialog::saveOptions()
     session->setIgnoreLimitsOnLAN(!m_ui->checkLimitLocalPeerRate->isChecked());
     pref->setSchedulerStartTime(m_ui->timeEditScheduleFrom->time());
     pref->setSchedulerEndTime(m_ui->timeEditScheduleTo->time());
-    pref->setSchedulerDays(static_cast<SchedulerDays>(m_ui->comboBoxScheduleDays->currentIndex()));
+    pref->setSchedulerDays(static_cast<Scheduler::Days>(m_ui->comboBoxScheduleDays->currentIndex()));
     session->setBandwidthSchedulerEnabled(m_ui->groupBoxSchedule->isChecked());
 
     auto proxyConfigManager = Net::ProxyConfigurationManager::instance();
@@ -798,12 +796,12 @@ void OptionsDialog::saveOptions()
     proxyConfigManager->setProxyOnlyForTorrents(m_ui->isProxyOnlyForTorrents->isChecked());
     proxyConfigManager->setProxyConfiguration(proxyConf);
 
-    session->setProxyPeerConnectionsEnabled(m_ui->checkProxyPeerConnecs->isChecked());
+    session->setProxyPeerConnectionsEnabled(m_ui->checkProxyPeerConnections->isChecked());
     // End Connection preferences
 
     // Bittorrent preferences
-    session->setMaxConnections(getMaxConnecs());
-    session->setMaxConnectionsPerTorrent(getMaxConnecsPerTorrent());
+    session->setMaxConnections(getMaxConnections());
+    session->setMaxConnectionsPerTorrent(getMaxConnectionsPerTorrent());
     session->setMaxUploads(getMaxUploads());
     session->setMaxUploadsPerTorrent(getMaxUploadsPerTorrent());
     session->setDHTEnabled(isDHTEnabled());
@@ -869,7 +867,7 @@ void OptionsDialog::saveOptions()
         pref->setWebUIHostHeaderValidationEnabled(m_ui->groupHostHeaderValidation->isChecked());
         // DynDNS
         pref->setDynDNSEnabled(m_ui->checkDynDNS->isChecked());
-        pref->setDynDNSService(m_ui->comboDNSService->currentIndex());
+        pref->setDynDNSService(static_cast<DNS::Service>(m_ui->comboDNSService->currentIndex()));
         pref->setDynDomainName(m_ui->domainNameTxt->text());
         pref->setDynDNSUsername(m_ui->DNSUsernameTxt->text());
         pref->setDynDNSPassword(m_ui->DNSPasswordTxt->text());
@@ -879,6 +877,9 @@ void OptionsDialog::saveOptions()
         // Custom HTTP headers
         pref->setWebUICustomHTTPHeadersEnabled(m_ui->groupWebUIAddCustomHTTPHeaders->isChecked());
         pref->setWebUICustomHTTPHeaders(m_ui->textWebUICustomHTTPHeaders->toPlainText());
+        // Reverse proxy
+        pref->setWebUIReverseProxySupportEnabled(m_ui->groupEnableReverseProxySupport->isChecked());
+        pref->setWebUITrustedReverseProxiesList(m_ui->textTrustedReverseProxiesList->text());
     }
     // End Web UI
     // End preferences
@@ -935,12 +936,12 @@ void OptionsDialog::loadOptions()
     m_ui->checkProgramAutoExitConfirm->setChecked(!pref->dontConfirmAutoExit());
 
 #ifndef Q_OS_MACOS
-    m_ui->checkShowSystray->setChecked(pref->systrayIntegration());
+    m_ui->checkShowSystray->setChecked(pref->systemTrayEnabled());
     if (m_ui->checkShowSystray->isChecked())
     {
         m_ui->checkMinimizeToSysTray->setChecked(pref->minimizeToTray());
         m_ui->checkCloseToSystray->setChecked(pref->closeToTray());
-        m_ui->comboTrayIcon->setCurrentIndex(pref->trayIconStyle());
+        m_ui->comboTrayIcon->setCurrentIndex(static_cast<int>(pref->trayIconStyle()));
     }
 #endif
 
@@ -996,16 +997,16 @@ void OptionsDialog::loadOptions()
     m_ui->deleteTorrentBox->setChecked(autoDeleteMode != TorrentFileGuard::Never);
     m_ui->deleteCancelledTorrentBox->setChecked(autoDeleteMode == TorrentFileGuard::Always);
 
-    m_ui->textSavePath->setSelectedPath(session->defaultSavePath());
+    m_ui->textSavePath->setSelectedPath(session->savePath());
     m_ui->checkUseSubcategories->setChecked(session->isSubcategoriesEnabled());
     m_ui->comboSavingMode->setCurrentIndex(!session->isAutoTMMDisabledByDefault());
     m_ui->comboTorrentCategoryChanged->setCurrentIndex(session->isDisableAutoTMMWhenCategoryChanged());
     m_ui->comboCategoryChanged->setCurrentIndex(session->isDisableAutoTMMWhenCategorySavePathChanged());
     m_ui->comboCategoryDefaultPathChanged->setCurrentIndex(session->isDisableAutoTMMWhenDefaultSavePathChanged());
-    m_ui->checkTempFolder->setChecked(session->isTempPathEnabled());
-    m_ui->textTempPath->setEnabled(m_ui->checkTempFolder->isChecked());
-    m_ui->textTempPath->setEnabled(m_ui->checkTempFolder->isChecked());
-    m_ui->textTempPath->setSelectedPath(Utils::Fs::toNativePath(session->tempPath()));
+    m_ui->checkUseDownloadPath->setChecked(session->isDownloadPathEnabled());
+    m_ui->textDownloadPath->setEnabled(m_ui->checkUseDownloadPath->isChecked());
+    m_ui->textDownloadPath->setEnabled(m_ui->checkUseDownloadPath->isChecked());
+    m_ui->textDownloadPath->setSelectedPath(Utils::Fs::toNativePath(session->downloadPath()));
     m_ui->checkAppendqB->setChecked(session->isAppendExtensionEnabled());
     m_ui->checkPreallocateAll->setChecked(session->isPreallocationEnabled());
     m_ui->checkRecursiveDownload->setChecked(!pref->recursiveDownloadDisabled());
@@ -1051,54 +1052,64 @@ void OptionsDialog::loadOptions()
 
     m_ui->autoRunBox->setChecked(pref->isAutoRunEnabled());
     m_ui->lineEditAutoRun->setText(pref->getAutoRunProgram());
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)) && defined(Q_OS_WIN)
+#if defined(Q_OS_WIN)
     m_ui->autoRunConsole->setChecked(pref->isAutoRunConsoleEnabled());
 #else
     m_ui->autoRunConsole->hide();
 #endif
-    intValue = pref->getActionOnDblClOnTorrentDl();
-    if (intValue >= m_ui->actionTorrentDlOnDblClBox->count())
-        intValue = 0;
-    m_ui->actionTorrentDlOnDblClBox->setCurrentIndex(intValue);
-    intValue = pref->getActionOnDblClOnTorrentFn();
-    if (intValue >= m_ui->actionTorrentFnOnDblClBox->count())
-        intValue = 1;
-    m_ui->actionTorrentFnOnDblClBox->setCurrentIndex(intValue);
+
+    m_ui->actionTorrentDlOnDblClBox->setItemData(0, TOGGLE_PAUSE);
+    m_ui->actionTorrentDlOnDblClBox->setItemData(1, OPEN_DEST);
+    m_ui->actionTorrentDlOnDblClBox->setItemData(2, PREVIEW_FILE);
+    m_ui->actionTorrentDlOnDblClBox->setItemData(3, SHOW_OPTIONS);
+    m_ui->actionTorrentDlOnDblClBox->setItemData(4, NO_ACTION);
+    int actionDownloading = pref->getActionOnDblClOnTorrentDl();
+    if ((actionDownloading < 0) || (actionDownloading >= m_ui->actionTorrentDlOnDblClBox->count()))
+        actionDownloading = TOGGLE_PAUSE;
+    m_ui->actionTorrentDlOnDblClBox->setCurrentIndex(m_ui->actionTorrentDlOnDblClBox->findData(actionDownloading));
+
+    m_ui->actionTorrentFnOnDblClBox->setItemData(0, TOGGLE_PAUSE);
+    m_ui->actionTorrentFnOnDblClBox->setItemData(1, OPEN_DEST);
+    m_ui->actionTorrentFnOnDblClBox->setItemData(2, PREVIEW_FILE);
+    m_ui->actionTorrentFnOnDblClBox->setItemData(3, SHOW_OPTIONS);
+    m_ui->actionTorrentFnOnDblClBox->setItemData(4, NO_ACTION);
+    int actionSeeding = pref->getActionOnDblClOnTorrentFn();
+    if ((actionSeeding < 0) || (actionSeeding >= m_ui->actionTorrentFnOnDblClBox->count()))
+        actionSeeding = OPEN_DEST;
+    m_ui->actionTorrentFnOnDblClBox->setCurrentIndex(m_ui->actionTorrentFnOnDblClBox->findData(actionSeeding));
     // End Downloads preferences
 
     // Connection preferences
     m_ui->comboProtocol->setCurrentIndex(static_cast<int>(session->btProtocol()));
-    m_ui->checkUPnP->setChecked(Net::PortForwarder::instance()->isEnabled());
-    m_ui->checkRandomPort->setChecked(session->useRandomPort());
     m_ui->spinPort->setValue(session->port());
-    m_ui->spinPort->setDisabled(m_ui->checkRandomPort->isChecked());
+    m_ui->checkUPnP->setChecked(Net::PortForwarder::instance()->isEnabled());
 
     intValue = session->maxConnections();
     if (intValue > 0)
     {
         // enable
-        m_ui->checkMaxConnecs->setChecked(true);
+        m_ui->checkMaxConnections->setChecked(true);
         m_ui->spinMaxConnec->setEnabled(true);
         m_ui->spinMaxConnec->setValue(intValue);
     }
     else
     {
         // disable
-        m_ui->checkMaxConnecs->setChecked(false);
+        m_ui->checkMaxConnections->setChecked(false);
         m_ui->spinMaxConnec->setEnabled(false);
     }
     intValue = session->maxConnectionsPerTorrent();
     if (intValue > 0)
     {
         // enable
-        m_ui->checkMaxConnecsPerTorrent->setChecked(true);
+        m_ui->checkMaxConnectionsPerTorrent->setChecked(true);
         m_ui->spinMaxConnecPerTorrent->setEnabled(true);
         m_ui->spinMaxConnecPerTorrent->setValue(intValue);
     }
     else
     {
         // disable
-        m_ui->checkMaxConnecsPerTorrent->setChecked(false);
+        m_ui->checkMaxConnectionsPerTorrent->setChecked(false);
         m_ui->spinMaxConnecPerTorrent->setEnabled(false);
     }
     intValue = session->maxUploads();
@@ -1163,7 +1174,7 @@ void OptionsDialog::loadOptions()
     m_ui->textProxyUsername->setText(proxyConf.username);
     m_ui->textProxyPassword->setText(proxyConf.password);
 
-    m_ui->checkProxyPeerConnecs->setChecked(session->isProxyPeerConnectionsEnabled());
+    m_ui->checkProxyPeerConnections->setChecked(session->isProxyPeerConnectionsEnabled());
     m_ui->isProxyOnlyForTorrents->setChecked(proxyConfigManager->isProxyOnlyForTorrents());
     enableProxy(m_ui->comboProxyType->currentIndex());
 
@@ -1282,6 +1293,9 @@ void OptionsDialog::loadOptions()
     // Custom HTTP headers
     m_ui->groupWebUIAddCustomHTTPHeaders->setChecked(pref->isWebUICustomHTTPHeadersEnabled());
     m_ui->textWebUICustomHTTPHeaders->setPlainText(pref->getWebUICustomHTTPHeaders());
+    // Reverse proxy
+    m_ui->groupEnableReverseProxySupport->setChecked(pref->isWebUIReverseProxySupportEnabled());
+    m_ui->textTrustedReverseProxiesList->setText(pref->getWebUITrustedReverseProxiesList());
     // End Web UI preferences
 }
 
@@ -1344,10 +1358,11 @@ bool OptionsDialog::startMinimized() const
 }
 
 #ifndef Q_OS_MACOS
-bool OptionsDialog::systrayIntegration() const
+bool OptionsDialog::systemTrayEnabled() const
 {
-    if (!QSystemTrayIcon::isSystemTrayAvailable()) return false;
-    return m_ui->checkShowSystray->isChecked();
+    return QSystemTrayIcon::isSystemTrayAvailable()
+        ? m_ui->checkShowSystray->isChecked()
+        : false;
 }
 
 bool OptionsDialog::minimizeToTray() const
@@ -1380,17 +1395,17 @@ int OptionsDialog::getMaxSeedingMinutes() const
 }
 
 // Return max connections number
-int OptionsDialog::getMaxConnecs() const
+int OptionsDialog::getMaxConnections() const
 {
-    if (!m_ui->checkMaxConnecs->isChecked())
+    if (!m_ui->checkMaxConnections->isChecked())
         return -1;
 
     return m_ui->spinMaxConnec->value();
 }
 
-int OptionsDialog::getMaxConnecsPerTorrent() const
+int OptionsDialog::getMaxConnectionsPerTorrent() const
 {
-    if (!m_ui->checkMaxConnecsPerTorrent->isChecked())
+    if (!m_ui->checkMaxConnectionsPerTorrent->isChecked())
         return -1;
 
     return m_ui->spinMaxConnecPerTorrent->value();
@@ -1439,27 +1454,24 @@ void OptionsDialog::on_buttonBox_accepted()
     accept();
 }
 
-void OptionsDialog::applySettings(QAbstractButton *button)
+void OptionsDialog::applySettings()
 {
-    if (button == m_applyButton)
+    if (!schedTimesOk())
     {
-        if (!schedTimesOk())
-        {
-            m_ui->tabSelection->setCurrentRow(TAB_SPEED);
-            return;
-        }
-        if (!webUIAuthenticationOk())
-        {
-            m_ui->tabSelection->setCurrentRow(TAB_WEBUI);
-            return;
-        }
-        if (!isAlternativeWebUIPathValid())
-        {
-            m_ui->tabSelection->setCurrentRow(TAB_WEBUI);
-            return;
-        }
-        saveOptions();
+        m_ui->tabSelection->setCurrentRow(TAB_SPEED);
+        return;
     }
+    if (!webUIAuthenticationOk())
+    {
+        m_ui->tabSelection->setCurrentRow(TAB_WEBUI);
+        return;
+    }
+    if (!isAlternativeWebUIPathValid())
+    {
+        m_ui->tabSelection->setCurrentRow(TAB_WEBUI);
+        return;
+    }
+    saveOptions();
 }
 
 void OptionsDialog::closeEvent(QCloseEvent *e)
@@ -1499,7 +1511,7 @@ void OptionsDialog::enableProxy(const int index)
         m_ui->textProxyIP->setEnabled(true);
         m_ui->lblProxyPort->setEnabled(true);
         m_ui->spinProxyPort->setEnabled(true);
-        m_ui->checkProxyPeerConnecs->setEnabled(true);
+        m_ui->checkProxyPeerConnections->setEnabled(true);
         if (index >= 2)
         { // SOCKS5 or HTTP
             m_ui->checkProxyAuth->setEnabled(true);
@@ -1519,7 +1531,7 @@ void OptionsDialog::enableProxy(const int index)
         m_ui->textProxyIP->setEnabled(false);
         m_ui->lblProxyPort->setEnabled(false);
         m_ui->spinProxyPort->setEnabled(false);
-        m_ui->checkProxyPeerConnecs->setEnabled(false);
+        m_ui->checkProxyPeerConnections->setEnabled(false);
         m_ui->isProxyOnlyForTorrents->setEnabled(false);
         m_ui->checkProxyAuth->setEnabled(false);
     }
@@ -1643,73 +1655,85 @@ QString OptionsDialog::getFinishedTorrentExportDir() const
     return {};
 }
 
-// Return action on double-click on a downloading torrent set in options
-int OptionsDialog::getActionOnDblClOnTorrentDl() const
-{
-    if (m_ui->actionTorrentDlOnDblClBox->currentIndex() < 1)
-        return 0;
-    return m_ui->actionTorrentDlOnDblClBox->currentIndex();
-}
-
-// Return action on double-click on a finished torrent set in options
-int OptionsDialog::getActionOnDblClOnTorrentFn() const
-{
-    if (m_ui->actionTorrentFnOnDblClBox->currentIndex() < 1)
-        return 0;
-    return m_ui->actionTorrentFnOnDblClBox->currentIndex();
-}
-
-void OptionsDialog::on_addScanFolderButton_clicked()
+void OptionsDialog::on_addWatchedFolderButton_clicked()
 {
     Preferences *const pref = Preferences::instance();
     const QString dir = QFileDialog::getExistingDirectory(this, tr("Select folder to monitor"),
-                                                          Utils::Fs::toNativePath(Utils::Fs::folderName(pref->getScanDirsLastPath())));
-    if (!dir.isEmpty())
+        Utils::Fs::toNativePath(Utils::Fs::folderName(pref->getScanDirsLastPath())));
+    if (dir.isEmpty())
+        return;
+
+    auto dialog = new WatchedFolderOptionsDialog({}, this);
+    dialog->setModal(true);
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    connect(dialog, &QDialog::accepted, this, [this, dialog, dir, pref]()
     {
-        const ScanFoldersModel::PathStatus status = ScanFoldersModel::instance()->addPath(dir, ScanFoldersModel::DEFAULT_LOCATION, QString(), false);
-        QString error;
-        switch (status)
+        try
         {
-        case ScanFoldersModel::AlreadyInList:
-            error = tr("Folder is already being monitored:");
-            break;
-        case ScanFoldersModel::DoesNotExist:
-            error = tr("Folder does not exist:");
-            break;
-        case ScanFoldersModel::CannotRead:
-            error = tr("Folder is not readable:");
-            break;
-        default:
+            auto watchedFoldersModel = static_cast<WatchedFoldersModel *>(m_ui->scanFoldersView->model());
+            watchedFoldersModel->addFolder(dir, dialog->watchedFolderOptions());
+
             pref->setScanDirsLastPath(dir);
-            m_addedScanDirs << dir;
-            for (int i = 0; i < ScanFoldersModel::instance()->columnCount(); ++i)
+
+            for (int i = 0; i < watchedFoldersModel->columnCount(); ++i)
                 m_ui->scanFoldersView->resizeColumnToContents(i);
+
             enableApplyButton();
         }
+        catch (const RuntimeError &err)
+        {
+            QMessageBox::critical(this, tr("Adding entry failed"), err.message());
+        }
+    });
 
-        if (!error.isEmpty())
-            QMessageBox::critical(this, tr("Adding entry failed"), QString::fromLatin1("%1\n%2").arg(error, dir));
-    }
+    dialog->open();
 }
 
-void OptionsDialog::on_removeScanFolderButton_clicked()
+void OptionsDialog::on_editWatchedFolderButton_clicked()
+{
+    const QModelIndex selected
+        = m_ui->scanFoldersView->selectionModel()->selectedIndexes().at(0);
+
+    editWatchedFolderOptions(selected);
+}
+
+void OptionsDialog::on_removeWatchedFolderButton_clicked()
 {
     const QModelIndexList selected
         = m_ui->scanFoldersView->selectionModel()->selectedIndexes();
-    if (selected.isEmpty())
-        return;
-    Q_ASSERT(selected.count() == ScanFoldersModel::instance()->columnCount());
+
     for (const QModelIndex &index : selected)
-    {
-        if (index.column() == ScanFoldersModel::WATCH)
-            m_removedScanDirs << index.data().toString();
-    }
-    ScanFoldersModel::instance()->removePath(selected.first().row(), false);
+        m_ui->scanFoldersView->model()->removeRow(index.row());
 }
 
-void OptionsDialog::handleScanFolderViewSelectionChanged()
+void OptionsDialog::handleWatchedFolderViewSelectionChanged()
 {
-    m_ui->removeScanFolderButton->setEnabled(!m_ui->scanFoldersView->selectionModel()->selectedIndexes().isEmpty());
+    const QModelIndexList selectedIndexes = m_ui->scanFoldersView->selectionModel()->selectedIndexes();
+    m_ui->removeWatchedFolderButton->setEnabled(!selectedIndexes.isEmpty());
+    m_ui->editWatchedFolderButton->setEnabled(selectedIndexes.count() == 1);
+}
+
+void OptionsDialog::editWatchedFolderOptions(const QModelIndex &index)
+{
+    if (!index.isValid())
+        return;
+
+    auto watchedFoldersModel = static_cast<WatchedFoldersModel *>(m_ui->scanFoldersView->model());
+    auto dialog = new WatchedFolderOptionsDialog(watchedFoldersModel->folderOptions(index.row()), this);
+    dialog->setModal(true);
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    connect(dialog, &QDialog::accepted, this, [this, dialog, index, watchedFoldersModel]()
+    {
+        if (index.isValid())
+        {
+            // The index could be invalidated while the dialog was displayed,
+            // for example, if you deleted the folder using the Web API.
+            watchedFoldersModel->setFolderOptions(index.row(), dialog->watchedFolderOptions());
+            enableApplyButton();
+        }
+    });
+
+    dialog->open();
 }
 
 QString OptionsDialog::askForExportDir(const QString &currentExportPath)
@@ -1805,7 +1829,8 @@ void OptionsDialog::showConnectionTab()
 
 void OptionsDialog::on_registerDNSBtn_clicked()
 {
-    QDesktopServices::openUrl(Net::DNSUpdater::getRegistrationUrl(m_ui->comboDNSService->currentIndex()));
+    const auto service = static_cast<DNS::Service>(m_ui->comboDNSService->currentIndex());
+    QDesktopServices::openUrl(Net::DNSUpdater::getRegistrationUrl(service));
 }
 
 void OptionsDialog::on_IpFilterRefreshBtn_clicked()
