@@ -37,7 +37,7 @@
 #include <QTableView>
 
 #include "base/bittorrent/common.h"
-#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/torrent.h"
 #include "base/preferences.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
@@ -47,12 +47,16 @@
 
 #define SETTINGS_KEY(name) "PreviewSelectDialog/" name
 
-PreviewSelectDialog::PreviewSelectDialog(QWidget *parent, const BitTorrent::TorrentHandle *torrent)
+PreviewSelectDialog::PreviewSelectDialog(QWidget *parent, const BitTorrent::Torrent *torrent)
     : QDialog(parent)
     , m_ui(new Ui::PreviewSelectDialog)
     , m_torrent(torrent)
-    , m_storeDialogSize(SETTINGS_KEY("Dimension"))
+    , m_storeDialogSize(SETTINGS_KEY("Size"))
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    , m_storeTreeHeaderState("GUI/Qt6/" SETTINGS_KEY("HeaderState"))
+#else
     , m_storeTreeHeaderState(SETTINGS_KEY("HeaderState"))
+#endif
 {
     m_ui->setupUi(this);
 
@@ -85,14 +89,12 @@ PreviewSelectDialog::PreviewSelectDialog(QWidget *parent, const BitTorrent::Torr
     m_ui->previewList->setAlternatingRowColors(pref->useAlternatingRowColors());
     // Fill list in
     const QVector<qreal> fp = torrent->filesProgress();
-    int nbFiles = torrent->filesCount();
-    for (int i = 0; i < nbFiles; ++i)
+    for (int i = 0; i < torrent->filesCount(); ++i)
     {
-        QString fileName = torrent->fileName(i);
+        QString fileName = Utils::Fs::fileName(torrent->filePath(i));
         if (fileName.endsWith(QB_EXT))
-            fileName.chop(4);
-        QString extension = Utils::Fs::fileExtension(fileName).toUpper();
-        if (Utils::Misc::isPreviewable(extension))
+            fileName.chop(QB_EXT.length());
+        if (Utils::Misc::isPreviewable(fileName))
         {
             int row = m_previewListModel->rowCount();
             m_previewListModel->insertRow(row);
@@ -159,7 +161,7 @@ void PreviewSelectDialog::loadWindowState()
     Utils::Gui::resize(this, m_storeDialogSize);
 
     // Restore TreeView Header state
-    if (!m_storeTreeHeaderState.value().isEmpty())
+    if (!m_storeTreeHeaderState.get().isEmpty())
     {
         m_headerStateInitialized = m_ui->previewList->header()->restoreState(m_storeTreeHeaderState);
     }
