@@ -28,7 +28,6 @@
 
 #include "propertieswidget.h"
 
-#include <QAction>
 #include <QClipboard>
 #include <QDateTime>
 #include <QDebug>
@@ -82,6 +81,9 @@ PropertiesWidget::PropertiesWidget(QWidget *parent)
 
     m_state = VISIBLE;
 
+    // Files list
+    m_ui->filesList->header()->setContextMenuPolicy(Qt::CustomContextMenu);
+
     // Set Properties list model
     m_propListModel = new TorrentContentFilterModel(this);
     m_ui->filesList->setModel(m_propListModel);
@@ -109,6 +111,7 @@ PropertiesWidget::PropertiesWidget(QWidget *parent)
             , m_ui->filesList, qOverload<const QModelIndex &>(&QAbstractItemView::edit));
     connect(m_ui->filesList, &QWidget::customContextMenuRequested, this, &PropertiesWidget::displayFilesListMenu);
     connect(m_ui->filesList, &QAbstractItemView::doubleClicked, this, &PropertiesWidget::openItem);
+    connect(m_ui->filesList->header(), &QWidget::customContextMenuRequested, this, &PropertiesWidget::displayFileListHeaderMenu);
     connect(m_ui->filesList->header(), &QHeaderView::sectionMoved, this, &PropertiesWidget::saveSettings);
     connect(m_ui->filesList->header(), &QHeaderView::sectionResized, this, &PropertiesWidget::saveSettings);
     connect(m_ui->filesList->header(), &QHeaderView::sortIndicatorChanged, this, &PropertiesWidget::saveSettings);
@@ -174,6 +177,33 @@ PropertiesWidget::~PropertiesWidget()
     delete m_ui;
 }
 
+void PropertiesWidget::displayFileListHeaderMenu()
+{
+    QMenu *menu = new QMenu(this);
+    menu->setAttribute(Qt::WA_DeleteOnClose);
+
+    for (int i = 0; i < TorrentContentModelItem::TreeItemColumns::NB_COL; ++i)
+    {
+        QAction *myAct = menu->addAction(m_propListModel->headerData(i, Qt::Horizontal, Qt::DisplayRole).toString());
+        myAct->setCheckable(true);
+        myAct->setChecked(!m_ui->filesList->isColumnHidden(i));
+        if (i == TorrentContentModelItem::TreeItemColumns::COL_NAME)
+            myAct->setEnabled(false);
+
+        connect(myAct, &QAction::toggled, this, [this, i](const bool checked)
+        {
+            m_ui->filesList->setColumnHidden(i, !checked);
+
+            if (!m_ui->filesList->isColumnHidden(i) && (m_ui->filesList->columnWidth(i) <= 5))
+                m_ui->filesList->resizeColumnToContents(i);
+
+            saveSettings();
+        });
+    }
+
+    menu->popup(QCursor::pos());
+}
+
 void PropertiesWidget::showPiecesAvailability(bool show)
 {
     m_ui->labelPiecesAvailability->setVisible(show);
@@ -233,7 +263,8 @@ void PropertiesWidget::clear()
     m_ui->labelSavePathVal->clear();
     m_ui->labelCreatedOnVal->clear();
     m_ui->labelTotalPiecesVal->clear();
-    m_ui->labelHashVal->clear();
+    m_ui->labelInfohash1Val->clear();
+    m_ui->labelInfohash2Val->clear();
     m_ui->labelCommentVal->clear();
     m_ui->labelProgressVal->clear();
     m_ui->labelAverageAvailabilityVal->clear();
@@ -313,8 +344,9 @@ void PropertiesWidget::loadTorrentInfos(BitTorrent::Torrent *const torrent)
 
     // Save path
     updateSavePath(m_torrent);
-    // Hash
-    m_ui->labelHashVal->setText(m_torrent->hash());
+    // Info hashes
+    m_ui->labelInfohash1Val->setText(m_torrent->infoHash().v1().isValid() ? m_torrent->infoHash().v1().toString() : tr("N/A"));
+    m_ui->labelInfohash2Val->setText(m_torrent->infoHash().v2().isValid() ? m_torrent->infoHash().v2().toString() : tr("N/A"));
     m_propListModel->model()->clear();
     if (m_torrent->hasMetadata())
     {
@@ -330,20 +362,6 @@ void PropertiesWidget::loadTorrentInfos(BitTorrent::Torrent *const torrent)
         loadUrlSeeds();
 
         m_ui->labelCreatedByVal->setText(m_torrent->creator());
-
-        // List files in torrent
-        m_propListModel->model()->setupModelData(m_torrent->info());
-
-        // Expand single-item folders recursively
-        QModelIndex currentIndex;
-        while (m_propListModel->rowCount(currentIndex) == 1)
-        {
-            currentIndex = m_propListModel->index(0, 0, currentIndex);
-            m_ui->filesList->setExpanded(currentIndex, true);
-        }
-
-        // Load file priorities
-        m_propListModel->model()->updateFilesPriorities(m_torrent->filePriorities());
     }
     // Load dynamic data
     loadDynamicData();
@@ -380,7 +398,7 @@ void PropertiesWidget::saveSettings()
         sizes = hSplitter->sizes();
     else
         sizes = m_slideSizes;
-    qDebug("Sizes: %d", sizes.size());
+
     if (sizes.size() == 2)
         pref->setPropSplitterSizes(QString::number(sizes.first()) + ',' + QString::number(sizes.last()));
     pref->setPropFileListState(m_ui->filesList->header()->saveState());
@@ -504,12 +522,40 @@ void PropertiesWidget::loadDynamicData()
         {
             qDebug("Updating priorities in files tab");
             m_ui->filesList->setUpdatesEnabled(false);
-            m_propListModel->model()->updateFilesProgress(m_torrent->filesProgress());
-            m_propListModel->model()->updateFilesAvailability(m_torrent->availableFileFractions());
-            // XXX: We don't update file priorities regularly for performance
-            // reasons. This means that priorities will not be updated if
-            // set from the Web UI.
-            // PropListModel->model()->updateFilesPriorities(h.file_priorities());
+
+            // Load torrent content if not yet done so
+            const bool isContentInitialized = m_propListModel->model()->hasIndex(0, 0);
+            if (!isContentInitialized)
+            {
+                // List files in torrent
+                m_propListModel->model()->setupModelData(*m_torrent);
+                // Load file priorities
+                m_propListModel->model()->updateFilesPriorities(m_torrent->filePriorities());
+                // Update file progress/availability
+                m_propListModel->model()->updateFilesProgress(m_torrent->filesProgress());
+                m_propListModel->model()->updateFilesAvailability(m_torrent->availableFileFractions());
+
+                // Expand single-item folders recursively.
+                // This will trigger sorting and filtering so do it after all relevant data is loaded.
+                QModelIndex currentIndex;
+                while (m_propListModel->rowCount(currentIndex) == 1)
+                {
+                    currentIndex = m_propListModel->index(0, 0, currentIndex);
+                    m_ui->filesList->setExpanded(currentIndex, true);
+                }
+            }
+            else
+            {
+                // Torrent content was loaded already, only make some updates
+
+                m_propListModel->model()->updateFilesProgress(m_torrent->filesProgress());
+                m_propListModel->model()->updateFilesAvailability(m_torrent->availableFileFractions());
+                // XXX: We don't update file priorities regularly for performance
+                // reasons. This means that priorities will not be updated if
+                // set from the Web UI.
+                // m_propListModel->model()->updateFilesPriorities(m_torrent->filePriorities());
+            }
+
             m_ui->filesList->setUpdatesEnabled(true);
         }
         break;
@@ -519,6 +565,9 @@ void PropertiesWidget::loadDynamicData()
 
 void PropertiesWidget::loadUrlSeeds()
 {
+    if (!m_torrent)
+        return;
+
     m_ui->listWebSeeds->clear();
     qDebug("Loading URL seeds");
     const QVector<QUrl> hcSeeds = m_torrent->urlSeeds();
@@ -532,11 +581,12 @@ void PropertiesWidget::loadUrlSeeds()
 
 QString PropertiesWidget::getFullPath(const QModelIndex &index) const
 {
+    const QDir saveDir {m_torrent->actualStorageLocation()};
+
     if (m_propListModel->itemType(index) == TorrentContentModelItem::FileType)
     {
         const int fileIdx = m_propListModel->getFileIndex(index);
         const QString filename {m_torrent->filePath(fileIdx)};
-        const QDir saveDir {m_torrent->savePath(true)};
         const QString fullPath {Utils::Fs::expandPath(saveDir.absoluteFilePath(filename))};
         return fullPath;
     }
@@ -547,7 +597,6 @@ QString PropertiesWidget::getFullPath(const QModelIndex &index) const
     for (QModelIndex modelIdx = m_propListModel->parent(nameIndex); modelIdx.isValid(); modelIdx = modelIdx.parent())
         folderPath.prepend(modelIdx.data().toString() + '/');
 
-    const QDir saveDir {m_torrent->savePath(true)};
     const QString fullPath {Utils::Fs::expandPath(saveDir.absoluteFilePath(folderPath))};
     return fullPath;
 }
@@ -586,57 +635,85 @@ void PropertiesWidget::displayFilesListMenu(const QPoint &)
     {
         const QModelIndex index = selectedRows[0];
 
-        const QAction *actOpen = menu->addAction(UIThemeManager::instance()->getIcon("folder-documents"), tr("Open"));
-        connect(actOpen, &QAction::triggered, this, [this, index]() { openItem(index); });
-
-        const QAction *actOpenContainingFolder = menu->addAction(UIThemeManager::instance()->getIcon("inode-directory"), tr("Open Containing Folder"));
-        connect(actOpenContainingFolder, &QAction::triggered, this, [this, index]() { openParentFolder(index); });
-
-        const QAction *actRename = menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Rename..."));
-        connect(actRename, &QAction::triggered, this, [this]() { m_ui->filesList->renameSelectedFile(*m_torrent); });
-
+        menu->addAction(UIThemeManager::instance()->getIcon("folder-documents"), tr("Open")
+            , this, [this, index]() { openItem(index); });
+        menu->addAction(UIThemeManager::instance()->getIcon("inode-directory"), tr("Open Containing Folder")
+            , this, [this, index]() { openParentFolder(index); });
+        menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Rename...")
+            , this, [this]() { m_ui->filesList->renameSelectedFile(*m_torrent); });
         menu->addSeparator();
     }
 
     if (!m_torrent->isSeed())
     {
-        QMenu *subMenu = menu->addMenu(tr("Priority"));
-
-        const auto applyPriorities = [this, selectedRows](const BitTorrent::DownloadPriority prio)
+        const auto applyPriorities = [this](const BitTorrent::DownloadPriority prio)
         {
+            const QModelIndexList selectedRows = m_ui->filesList->selectionModel()->selectedRows(0);
             for (const QModelIndex &index : selectedRows)
             {
-                m_propListModel->setData(
-                    m_propListModel->index(index.row(), PRIORITY, index.parent()), static_cast<int>(prio));
+                m_propListModel->setData(index.sibling(index.row(), PRIORITY)
+                    , static_cast<int>(prio));
             }
 
             // Save changes
-            filteredFilesChanged();
+            this->applyPriorities();
         };
 
-        connect(m_ui->actionNotDownloaded, &QAction::triggered, subMenu, [applyPriorities]()
+        QMenu *subMenu = menu->addMenu(tr("Priority"));
+
+        subMenu->addAction(tr("Do not download"), subMenu, [applyPriorities]()
         {
             applyPriorities(BitTorrent::DownloadPriority::Ignored);
         });
-        subMenu->addAction(m_ui->actionNotDownloaded);
-
-        connect(m_ui->actionNormal, &QAction::triggered, subMenu, [applyPriorities]()
+        subMenu->addAction(tr("Normal"), subMenu, [applyPriorities]()
         {
             applyPriorities(BitTorrent::DownloadPriority::Normal);
         });
-        subMenu->addAction(m_ui->actionNormal);
-
-        connect(m_ui->actionHigh, &QAction::triggered, subMenu, [applyPriorities]()
+        subMenu->addAction(tr("High"), subMenu, [applyPriorities]()
         {
             applyPriorities(BitTorrent::DownloadPriority::High);
         });
-        subMenu->addAction(m_ui->actionHigh);
-
-        connect(m_ui->actionMaximum, &QAction::triggered, subMenu, [applyPriorities]()
+        subMenu->addAction(tr("Maximum"), subMenu, [applyPriorities]()
         {
             applyPriorities(BitTorrent::DownloadPriority::Maximum);
         });
-        subMenu->addAction(m_ui->actionMaximum);
+        subMenu->addSeparator();
+        subMenu->addAction(tr("By shown file order"), subMenu, [this]()
+        {
+            // Equally distribute the selected items into groups and for each group assign
+            // a download priority that will apply to each item. The number of groups depends on how
+            // many "download priority" are available to be assigned
+
+            const QModelIndexList selectedRows = m_ui->filesList->selectionModel()->selectedRows(0);
+
+            const qsizetype priorityGroups = 3;
+            const auto priorityGroupSize = std::max<qsizetype>((selectedRows.length() / priorityGroups), 1);
+
+            for (qsizetype i = 0; i < selectedRows.length(); ++i)
+            {
+                auto priority = BitTorrent::DownloadPriority::Ignored;
+                switch (i / priorityGroupSize)
+                {
+                case 0:
+                    priority = BitTorrent::DownloadPriority::Maximum;
+                    break;
+                case 1:
+                    priority = BitTorrent::DownloadPriority::High;
+                    break;
+                default:
+                case 2:
+                    priority = BitTorrent::DownloadPriority::Normal;
+                    break;
+                }
+
+                const QModelIndex &index = selectedRows[i];
+                m_propListModel->setData(index.sibling(index.row(), PRIORITY)
+                    , static_cast<int>(priority));
+
+                // Save changes
+                this->applyPriorities();
+            }
+        });
     }
 
     // The selected torrent might have disappeared during exec()
@@ -660,21 +737,17 @@ void PropertiesWidget::displayWebSeedListMenu(const QPoint &)
     QMenu *menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
-    const QAction *actAdd = menu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("New Web seed"));
-    connect(actAdd, &QAction::triggered, this, &PropertiesWidget::askWebSeed);
+    menu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("New Web seed"), this, &PropertiesWidget::askWebSeed);
 
     if (!rows.isEmpty())
     {
-        const QAction *actDel = menu->addAction(UIThemeManager::instance()->getIcon("list-remove"), tr("Remove Web seed"));
-        connect(actDel, &QAction::triggered, this, &PropertiesWidget::deleteSelectedUrlSeeds);
-
+        menu->addAction(UIThemeManager::instance()->getIcon("list-remove"), tr("Remove Web seed")
+            , this, &PropertiesWidget::deleteSelectedUrlSeeds);
         menu->addSeparator();
-
-        const QAction *actCpy = menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy Web seed URL"));
-        connect(actCpy, &QAction::triggered, this, &PropertiesWidget::copySelectedWebSeedsToClipboard);
-
-        const QAction *actEdit = menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Edit Web seed URL"));
-        connect(actEdit, &QAction::triggered, this, &PropertiesWidget::editWebSeed);
+        menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy Web seed URL")
+            , this, &PropertiesWidget::copySelectedWebSeedsToClipboard);
+        menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Edit Web seed URL")
+            , this, &PropertiesWidget::editWebSeed);
     }
 
     menu->popup(QCursor::pos());
@@ -693,21 +766,31 @@ void PropertiesWidget::configure()
     // Speed widget
     if (Preferences::instance()->isSpeedWidgetEnabled())
     {
-        if (!m_speedWidget || !qobject_cast<SpeedWidget *>(m_speedWidget))
+        if (!qobject_cast<SpeedWidget *>(m_speedWidget))
         {
-            m_ui->speedLayout->removeWidget(m_speedWidget);
-            delete m_speedWidget;
-            m_speedWidget = new SpeedWidget {this};
+            if (m_speedWidget)
+            {
+                m_ui->speedLayout->removeWidget(m_speedWidget);
+                delete m_speedWidget;
+            }
+
+            m_speedWidget = new SpeedWidget(this);
             m_ui->speedLayout->addWidget(m_speedWidget);
         }
     }
     else
     {
-        if (!m_speedWidget || !qobject_cast<QLabel *>(m_speedWidget))
+        if (!qobject_cast<QLabel *>(m_speedWidget))
         {
-            m_ui->speedLayout->removeWidget(m_speedWidget);
-            delete m_speedWidget;
-            auto *label = new QLabel(tr("<center><b>Speed graphs are disabled</b><p>You may change this setting in Advanced Options </center>"), this);
+            if (m_speedWidget)
+            {
+                m_ui->speedLayout->removeWidget(m_speedWidget);
+                delete m_speedWidget;
+            }
+
+            const auto displayText = QString::fromLatin1("<center><b>%1</b><p>%2</p></center>")
+                .arg(tr("Speed graphs are disabled"), tr("You can enable it in Advanced Options"));
+            auto *label = new QLabel(displayText, this);
             label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
             m_speedWidget = label;
             m_ui->speedLayout->addWidget(m_speedWidget);
@@ -780,7 +863,7 @@ void PropertiesWidget::editWebSeed()
 
     if (!m_ui->listWebSeeds->findItems(newSeed, Qt::MatchFixedString).empty())
     {
-        QMessageBox::warning(this, tr("qBittorrent"),
+        QMessageBox::warning(this, QLatin1String("qBittorrent"),
                              tr("This URL seed is already in the list."),
                              QMessageBox::Ok);
         return;
@@ -804,7 +887,8 @@ void PropertiesWidget::filteredFilesChanged()
 
 void PropertiesWidget::filterText(const QString &filter)
 {
-    m_propListModel->setFilterRegExp(QRegExp(filter, Qt::CaseInsensitive, QRegExp::WildcardUnix));
+    const QString pattern = Utils::String::wildcardToRegexPattern(filter);
+    m_propListModel->setFilterRegularExpression(QRegularExpression(pattern, QRegularExpression::CaseInsensitiveOption));
     if (filter.isEmpty())
     {
         m_ui->filesList->collapseAll();
