@@ -33,7 +33,6 @@
 #include <QClipboard>
 #include <QColor>
 #include <QDebug>
-#include <QHash>
 #include <QHeaderView>
 #include <QMenu>
 #include <QMessageBox>
@@ -46,7 +45,7 @@
 
 #include "base/bittorrent/peerinfo.h"
 #include "base/bittorrent/session.h"
-#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/torrent.h"
 #include "base/bittorrent/trackerentry.h"
 #include "base/global.h"
 #include "base/preferences.h"
@@ -72,6 +71,7 @@ TrackerListWidget::TrackerListWidget(PropertiesWidget *properties)
     setItemsExpandable(false);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     header()->setStretchLastSection(false); // Must be set after loadSettings() in order to work
+    header()->setTextElideMode(Qt::ElideRight);
     // Ensure that at least one column is visible at all times
     if (visibleColumnsCount() == 0)
         setColumnHidden(COL_URL, false);
@@ -172,7 +172,7 @@ void TrackerListWidget::setRowColor(const int row, const QColor &color)
 
 void TrackerListWidget::moveSelectionUp()
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent)
     {
         clear();
@@ -205,9 +205,7 @@ void TrackerListWidget::moveSelectionUp()
     for (int i = NB_STICKY_ITEM; i < topLevelItemCount(); ++i)
     {
         const QString trackerURL = topLevelItem(i)->data(COL_URL, Qt::DisplayRole).toString();
-        BitTorrent::TrackerEntry e(trackerURL);
-        e.setTier(i - NB_STICKY_ITEM);
-        trackers.append(e);
+        trackers.append({trackerURL, (i - NB_STICKY_ITEM)});
     }
 
     torrent->replaceTrackers(trackers);
@@ -218,7 +216,7 @@ void TrackerListWidget::moveSelectionUp()
 
 void TrackerListWidget::moveSelectionDown()
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent)
     {
         clear();
@@ -251,9 +249,7 @@ void TrackerListWidget::moveSelectionDown()
     for (int i = NB_STICKY_ITEM; i < topLevelItemCount(); ++i)
     {
         const QString trackerURL = topLevelItem(i)->data(COL_URL, Qt::DisplayRole).toString();
-        BitTorrent::TrackerEntry e(trackerURL);
-        e.setTier(i - NB_STICKY_ITEM);
-        trackers.append(e);
+        trackers.append({trackerURL, (i - NB_STICKY_ITEM)});
     }
 
     torrent->replaceTrackers(trackers);
@@ -281,28 +277,36 @@ void TrackerListWidget::clear()
     m_LSDItem->setText(COL_MSG, "");
 }
 
-void TrackerListWidget::loadStickyItems(const BitTorrent::TorrentHandle *torrent)
+void TrackerListWidget::loadStickyItems(const BitTorrent::Torrent *torrent)
 {
-    QString working = tr("Working");
-    QString disabled = tr("Disabled");
+    const QString working {tr("Working")};
+    const QString disabled {tr("Disabled")};
+    const QString torrentDisabled {tr("Disabled for this torrent")};
+    const auto *session = BitTorrent::Session::instance();
 
     // load DHT information
-    if (BitTorrent::Session::instance()->isDHTEnabled() && !torrent->isPrivate())
-        m_DHTItem->setText(COL_STATUS, working);
-    else
+    if (torrent->isPrivate() || torrent->isDHTDisabled())
+        m_DHTItem->setText(COL_STATUS, torrentDisabled);
+    else if (!session->isDHTEnabled())
         m_DHTItem->setText(COL_STATUS, disabled);
+    else
+        m_DHTItem->setText(COL_STATUS, working);
 
     // Load PeX Information
-    if (BitTorrent::Session::instance()->isPeXEnabled() && !torrent->isPrivate())
-        m_PEXItem->setText(COL_STATUS, working);
-    else
+    if (torrent->isPrivate() || torrent->isPEXDisabled())
+        m_PEXItem->setText(COL_STATUS, torrentDisabled);
+    else if (!session->isPeXEnabled())
         m_PEXItem->setText(COL_STATUS, disabled);
+    else
+        m_PEXItem->setText(COL_STATUS, working);
 
     // Load LSD Information
-    if (BitTorrent::Session::instance()->isLSDEnabled() && !torrent->isPrivate())
-        m_LSDItem->setText(COL_STATUS, working);
-    else
+    if (torrent->isPrivate() || torrent->isLSDDisabled())
+        m_LSDItem->setText(COL_STATUS, torrentDisabled);
+    else if (!session->isLSDEnabled())
         m_LSDItem->setText(COL_STATUS, disabled);
+    else
+        m_LSDItem->setText(COL_STATUS, working);
 
     if (torrent->isPrivate())
     {
@@ -353,24 +357,24 @@ void TrackerListWidget::loadStickyItems(const BitTorrent::TorrentHandle *torrent
 void TrackerListWidget::loadTrackers()
 {
     // Load trackers from torrent handle
-    const BitTorrent::TorrentHandle *torrent = m_properties->getCurrentTorrent();
+    const BitTorrent::Torrent *torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
     loadStickyItems(torrent);
 
     // Load actual trackers information
-    const QHash<QString, BitTorrent::TrackerInfo> trackerData = torrent->trackerInfos();
     QStringList oldTrackerURLs = m_trackerItems.keys();
 
     for (const BitTorrent::TrackerEntry &entry : asConst(torrent->trackers()))
     {
-        const QString trackerURL = entry.url();
+        const QString trackerURL = entry.url;
 
         QTreeWidgetItem *item = m_trackerItems.value(trackerURL, nullptr);
         if (!item)
         {
             item = new QTreeWidgetItem();
             item->setText(COL_URL, trackerURL);
+            item->setToolTip(COL_URL, trackerURL);
             addTopLevelItem(item);
             m_trackerItems[trackerURL] = item;
         }
@@ -379,39 +383,37 @@ void TrackerListWidget::loadTrackers()
             oldTrackerURLs.removeOne(trackerURL);
         }
 
-        item->setText(COL_TIER, QString::number(entry.tier()));
+        item->setText(COL_TIER, QString::number(entry.tier));
 
-        const BitTorrent::TrackerInfo data = trackerData.value(trackerURL);
-
-        switch (entry.status())
+        switch (entry.status)
         {
         case BitTorrent::TrackerEntry::Working:
             item->setText(COL_STATUS, tr("Working"));
-            item->setText(COL_MSG, "");
             break;
         case BitTorrent::TrackerEntry::Updating:
             item->setText(COL_STATUS, tr("Updating..."));
-            item->setText(COL_MSG, "");
             break;
         case BitTorrent::TrackerEntry::NotWorking:
             item->setText(COL_STATUS, tr("Not working"));
-            item->setText(COL_MSG, data.lastMessage.trimmed());
             break;
         case BitTorrent::TrackerEntry::NotContacted:
             item->setText(COL_STATUS, tr("Not contacted yet"));
-            item->setText(COL_MSG, "");
             break;
         }
 
-        item->setText(COL_PEERS, QString::number(data.numPeers));
-        item->setText(COL_SEEDS, ((entry.numSeeds() > -1)
-            ? QString::number(entry.numSeeds())
+        item->setText(COL_MSG, entry.message);
+        item->setToolTip(COL_MSG, entry.message);
+        item->setText(COL_PEERS, ((entry.numPeers > -1)
+            ? QString::number(entry.numPeers)
             : tr("N/A")));
-        item->setText(COL_LEECHES, ((entry.numLeeches() > -1)
-            ? QString::number(entry.numLeeches())
+        item->setText(COL_SEEDS, ((entry.numSeeds > -1)
+            ? QString::number(entry.numSeeds)
             : tr("N/A")));
-        item->setText(COL_DOWNLOADED, ((entry.numDownloaded() > -1)
-            ? QString::number(entry.numDownloaded())
+        item->setText(COL_LEECHES, ((entry.numLeeches > -1)
+            ? QString::number(entry.numLeeches)
+            : tr("N/A")));
+        item->setText(COL_DOWNLOADED, ((entry.numDownloaded > -1)
+            ? QString::number(entry.numDownloaded)
             : tr("N/A")));
 
         const Qt::Alignment alignment = (Qt::AlignRight | Qt::AlignVCenter);
@@ -430,12 +432,12 @@ void TrackerListWidget::loadTrackers()
 // Ask the user for new trackers and add them to the torrent
 void TrackerListWidget::askForTrackers()
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
     QVector<BitTorrent::TrackerEntry> trackers;
     for (const QString &tracker : asConst(TrackersAdditionDialog::askForTrackers(this, torrent)))
-        trackers << tracker;
+        trackers.append({tracker});
 
     torrent->addTrackers(trackers);
 }
@@ -458,7 +460,7 @@ void TrackerListWidget::copyTrackerUrl()
 
 void TrackerListWidget::deleteSelectedTrackers()
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent)
     {
         clear();
@@ -484,7 +486,7 @@ void TrackerListWidget::deleteSelectedTrackers()
 
     for (const BitTorrent::TrackerEntry &entry : trackers)
     {
-        if (!urlsToRemove.contains(entry.url()))
+        if (!urlsToRemove.contains(entry.url))
             remainingTrackers.push_back(entry);
     }
 
@@ -496,7 +498,7 @@ void TrackerListWidget::deleteSelectedTrackers()
 
 void TrackerListWidget::editSelectedTracker()
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
     const QVector<QTreeWidgetItem *> selectedTrackerItems = getSelectedTrackerItems();
@@ -521,18 +523,16 @@ void TrackerListWidget::editSelectedTracker()
     bool match = false;
     for (BitTorrent::TrackerEntry &entry : trackers)
     {
-        if (newTrackerURL == QUrl(entry.url()))
+        if (newTrackerURL == QUrl(entry.url))
         {
             QMessageBox::warning(this, tr("Tracker editing failed"), tr("The tracker URL already exists."));
             return;
         }
 
-        if (!match && (trackerURL == QUrl(entry.url())))
+        if (!match && (trackerURL == QUrl(entry.url)))
         {
             match = true;
-            BitTorrent::TrackerEntry newEntry(newTrackerURL.toString());
-            newEntry.setTier(entry.tier());
-            entry = newEntry;
+            entry.url = newTrackerURL.toString();
         }
     }
 
@@ -547,7 +547,7 @@ void TrackerListWidget::reannounceSelected()
     const QList<QTreeWidgetItem *> selItems = selectedItems();
     if (selItems.isEmpty()) return;
 
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
     const QVector<BitTorrent::TrackerEntry> trackers = torrent->trackers();
@@ -564,7 +564,7 @@ void TrackerListWidget::reannounceSelected()
         // Trackers case
         for (int i = 0; i < trackers.size(); ++i)
         {
-            if (item->text(COL_URL) == trackers[i].url())
+            if (item->text(COL_URL) == trackers[i].url)
             {
                 torrent->forceReannounce(i);
                 break;
@@ -577,39 +577,35 @@ void TrackerListWidget::reannounceSelected()
 
 void TrackerListWidget::showTrackerListMenu(const QPoint &)
 {
-    BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
     QMenu *menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
     // Add actions
-    const QAction *addAct = menu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("Add a new tracker..."));
-    connect(addAct, &QAction::triggered, this, &TrackerListWidget::askForTrackers);
+    menu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("Add a new tracker...")
+        , this, &TrackerListWidget::askForTrackers);
 
     if (!getSelectedTrackerItems().isEmpty())
     {
-        const QAction *editAct = menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"),tr("Edit tracker URL..."));
-        connect(editAct, &QAction::triggered, this, &TrackerListWidget::editSelectedTracker);
-
-        const QAction *delAct = menu->addAction(UIThemeManager::instance()->getIcon("list-remove"), tr("Remove tracker"));
-        connect(delAct, &QAction::triggered, this, &TrackerListWidget::deleteSelectedTrackers);
-
-        const QAction *copyAct = menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy tracker URL"));
-        connect(copyAct, &QAction::triggered, this, &TrackerListWidget::copyTrackerUrl);
+        menu->addAction(UIThemeManager::instance()->getIcon("edit-rename"),tr("Edit tracker URL...")
+            , this, &TrackerListWidget::editSelectedTracker);
+        menu->addAction(UIThemeManager::instance()->getIcon("list-remove"), tr("Remove tracker")
+            , this, &TrackerListWidget::deleteSelectedTrackers);
+        menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy tracker URL")
+            , this, &TrackerListWidget::copyTrackerUrl);
     }
 
     if (!torrent->isPaused())
     {
-        const QAction *reannounceSelAct = menu->addAction(UIThemeManager::instance()->getIcon("view-refresh"), tr("Force reannounce to selected trackers"));
-        connect(reannounceSelAct, &QAction::triggered, this, &TrackerListWidget::reannounceSelected);
-
+        menu->addAction(UIThemeManager::instance()->getIcon("view-refresh"), tr("Force reannounce to selected trackers")
+            , this, &TrackerListWidget::reannounceSelected);
         menu->addSeparator();
-
-        const QAction *reannounceAllAct = menu->addAction(UIThemeManager::instance()->getIcon("view-refresh"), tr("Force reannounce to all trackers"));
-        connect(reannounceAllAct, &QAction::triggered, this, [this]()
+        menu->addAction(UIThemeManager::instance()->getIcon("view-refresh"), tr("Force reannounce to all trackers")
+            , this, [this]()
         {
-            BitTorrent::TorrentHandle *h = m_properties->getCurrentTorrent();
+            BitTorrent::Torrent *h = m_properties->getCurrentTorrent();
             h->forceReannounce();
             h->forceDHTAnnounce();
         });
