@@ -246,10 +246,10 @@ void RSSWidget::askNewFolder()
                                   ? RSS::Session::instance()->rootFolder()
                                   : qobject_cast<RSS::Folder *>(m_feedListWidget->getRSSItem(destItem)));
 
-    QString error;
     const QString newFolderPath = RSS::Item::joinPath(rssDestFolder->path(), newName);
-    if (!RSS::Session::instance()->addFolder(newFolderPath, &error))
-        QMessageBox::warning(this, "qBittorrent", error, QMessageBox::Ok);
+    const nonstd::expected<void, QString> result = RSS::Session::instance()->addFolder(newFolderPath);
+    if (!result)
+        QMessageBox::warning(this, "qBittorrent", result.error(), QMessageBox::Ok);
 
     // Expand destination folder to display new feed
     if (destItem && (destItem != m_feedListWidget->stickyUnreadItem()))
@@ -287,11 +287,11 @@ void RSSWidget::on_newFeedButton_clicked()
                                   ? RSS::Session::instance()->rootFolder()
                                   : qobject_cast<RSS::Folder *>(m_feedListWidget->getRSSItem(destItem)));
 
-    QString error;
     // NOTE: We still add feed using legacy way (with URL as feed name)
     const QString newFeedPath = RSS::Item::joinPath(rssDestFolder->path(), newURL);
-    if (!RSS::Session::instance()->addFeed(newURL, newFeedPath, &error))
-        QMessageBox::warning(this, "qBittorrent", error, QMessageBox::Ok);
+    const nonstd::expected<void, QString> result = RSS::Session::instance()->addFeed(newURL, newFeedPath);
+    if (!result)
+        QMessageBox::warning(this, "qBittorrent", result.error(), QMessageBox::Ok);
 
     // Expand destination folder to display new feed
     if (destItem && (destItem != m_feedListWidget->stickyUnreadItem()))
@@ -411,10 +411,10 @@ void RSSWidget::renameSelectedRSSItem()
         // Check if name is already taken
         if (!ok) return;
 
-        QString error;
-        if (!RSS::Session::instance()->moveItem(rssItem, RSS::Item::joinPath(parentPath, newName), &error))
+        const nonstd::expected<void, QString> result = RSS::Session::instance()->moveItem(rssItem, RSS::Item::joinPath(parentPath, newName));
+        if (!result)
         {
-            QMessageBox::warning(nullptr, tr("Rename failed"), error);
+            QMessageBox::warning(nullptr, tr("Rename failed"), result.error());
             ok = false;
         }
     } while (!ok);
@@ -486,7 +486,7 @@ void RSSWidget::handleCurrentArticleItemChanged(QListWidgetItem *currentItem, QL
         QString::fromLatin1("<div style='border: 2px solid red; margin-left: 5px; margin-right: 5px; margin-bottom: 5px;'>") +
         QString::fromLatin1("<div style='background-color: \"%1\"; font-weight: bold; color: \"%2\";'>%3</div>").arg(highlightedBaseColor, highlightedBaseTextColor, article->title());
     if (article->date().isValid())
-        html += QString::fromLatin1("<div style='background-color: \"%1\";'><b>%2</b>%3</div>").arg(alternateBaseColor, tr("Date: "), article->date().toLocalTime().toString(Qt::SystemLocaleLongDate));
+        html += QString::fromLatin1("<div style='background-color: \"%1\";'><b>%2</b>%3</div>").arg(alternateBaseColor, tr("Date: "), QLocale::system().toString(article->date().toLocalTime()));
     if (!article->author().isEmpty())
         html += QString::fromLatin1("<div style='background-color: \"%1\";'><b>%2</b>%3</div>").arg(alternateBaseColor, tr("Author: "), article->author());
     html += "</div>"
