@@ -30,7 +30,6 @@
 
 #include <QDebug>
 #include <QIcon>
-#include <QSet>
 #include <QVector>
 
 #include "base/bittorrent/session.h"
@@ -315,7 +314,7 @@ TagModelItem *TagFilterModel::findItem(const QString &tag)
     return &m_tagItems[row];
 }
 
-QVector<TagModelItem *> TagFilterModel::findItems(const QSet<QString> &tags)
+QVector<TagModelItem *> TagFilterModel::findItems(const TagSet &tags)
 {
     QVector<TagModelItem *> items;
     items.reserve(tags.count());
