@@ -31,13 +31,15 @@
 #include <QAbstractListModel>
 #include <QtContainerFwd>
 
+#include "base/tagset.h"
+
 class QModelIndex;
 
 class TagModelItem;
 
 namespace BitTorrent
 {
-    class TorrentHandle;
+    class Torrent;
 }
 
 class TagFilterModel final : public QAbstractListModel
@@ -62,10 +64,10 @@ public:
 private slots:
     void tagAdded(const QString &tag);
     void tagRemoved(const QString &tag);
-    void torrentTagAdded(BitTorrent::TorrentHandle *const torrent, const QString &tag);
-    void torrentTagRemoved(BitTorrent::TorrentHandle *const, const QString &tag);
-    void torrentAdded(BitTorrent::TorrentHandle *const torrent);
-    void torrentAboutToBeRemoved(BitTorrent::TorrentHandle *const torrent);
+    void torrentTagAdded(BitTorrent::Torrent *const torrent, const QString &tag);
+    void torrentTagRemoved(BitTorrent::Torrent *const, const QString &tag);
+    void torrentAdded(BitTorrent::Torrent *const torrent);
+    void torrentAboutToBeRemoved(BitTorrent::Torrent *const torrent);
 
 private:
     static QString tagDisplayName(const QString &tag);
@@ -76,7 +78,7 @@ private:
     bool isValidRow(int row) const;
     int findRow(const QString &tag) const;
     TagModelItem *findItem(const QString &tag);
-    QVector<TagModelItem *> findItems(const QSet<QString> &tags);
+    QVector<TagModelItem *> findItems(const TagSet &tags);
     TagModelItem *allTagsItem();
     TagModelItem *untaggedItem();
 
