@@ -26,30 +26,28 @@
  * exception statement from your version.
  */
 
-#ifndef TORRENTCONTENTTREEVIEW_H
-#define TORRENTCONTENTTREEVIEW_H
+#pragma once
 
 #include <QTreeView>
 
 namespace BitTorrent
 {
-    class TorrentHandle;
+    class AbstractFileStorage;
+    class Torrent;
     class TorrentInfo;
 }
 
 class TorrentContentTreeView final : public QTreeView
 {
     Q_OBJECT
+    Q_DISABLE_COPY_MOVE(TorrentContentTreeView)
 
 public:
     explicit TorrentContentTreeView(QWidget *parent = nullptr);
     void keyPressEvent(QKeyEvent *event) override;
 
-    void renameSelectedFile(BitTorrent::TorrentHandle *torrent);
-    void renameSelectedFile(BitTorrent::TorrentInfo &torrent);
+    void renameSelectedFile(BitTorrent::AbstractFileStorage &fileStorage);
 
 private:
     QModelIndex currentNameCell();
 };
-
-#endif // TORRENTCONTENTTREEVIEW_H
