@@ -27,14 +27,14 @@
  * exception statement from your version.
  */
 
-#ifndef TRANSFERLISTMODEL_H
-#define TRANSFERLISTMODEL_H
+#pragma once
 
 #include <QAbstractListModel>
 #include <QColor>
+#include <QHash>
 #include <QList>
 
-#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/torrent.h"
 
 namespace BitTorrent
 {
@@ -44,7 +44,7 @@ namespace BitTorrent
 class TransferListModel final : public QAbstractListModel
 {
     Q_OBJECT
-    Q_DISABLE_COPY(TransferListModel)
+    Q_DISABLE_COPY_MOVE(TransferListModel)
 
 public:
     enum Column
@@ -99,27 +99,24 @@ public:
     QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
     Qt::ItemFlags flags(const QModelIndex &index) const override;
 
-    BitTorrent::TorrentHandle *torrentHandle(const QModelIndex &index) const;
-
-    void setStateForeground(BitTorrent::TorrentState state, const QColor& color);
-    QColor stateForeground(BitTorrent::TorrentState state) const;
+    BitTorrent::Torrent *torrentHandle(const QModelIndex &index) const;
 
 private slots:
-    void addTorrent(BitTorrent::TorrentHandle *const torrent);
-    void handleTorrentAboutToBeRemoved(BitTorrent::TorrentHandle *const torrent);
-    void handleTorrentStatusUpdated(BitTorrent::TorrentHandle *const torrent);
-    void handleTorrentsUpdated(const QVector<BitTorrent::TorrentHandle *> &torrents);
+    void addTorrent(BitTorrent::Torrent *const torrent);
+    void handleTorrentAboutToBeRemoved(BitTorrent::Torrent *const torrent);
+    void handleTorrentStatusUpdated(BitTorrent::Torrent *const torrent);
+    void handleTorrentsUpdated(const QVector<BitTorrent::Torrent *> &torrents);
 
 private:
     void configure();
-    QString displayValue(const BitTorrent::TorrentHandle *torrent, int column) const;
-    QVariant internalValue(const BitTorrent::TorrentHandle *torrent, int column, bool alt = false) const;
+    QString displayValue(const BitTorrent::Torrent *torrent, int column) const;
+    QVariant internalValue(const BitTorrent::Torrent *torrent, int column, bool alt) const;
 
-    QList<BitTorrent::TorrentHandle *> m_torrentList;  // maps row number to torrent handle
-    QHash<BitTorrent::TorrentHandle *, int> m_torrentMap;  // maps torrent handle to row number
+    QList<BitTorrent::Torrent *> m_torrentList;  // maps row number to torrent handle
+    QHash<BitTorrent::Torrent *, int> m_torrentMap;  // maps torrent handle to row number
     const QHash<BitTorrent::TorrentState, QString> m_statusStrings;
     // row text colors
-    QHash<BitTorrent::TorrentState, QColor> m_stateForegroundColors;
+    const QHash<BitTorrent::TorrentState, QColor> m_stateThemeColors;
 
     enum class HideZeroValuesMode
     {
@@ -130,5 +127,3 @@ private:
 
     HideZeroValuesMode m_hideZeroValuesMode = HideZeroValuesMode::Never;
 };
-
-#endif // TRANSFERLISTMODEL_H
