@@ -29,7 +29,10 @@
 #pragma once
 
 #include <QSortFilterProxyModel>
+
+#include "base/settingvalue.h"
 #include "base/torrentfilter.h"
+#include "base/utils/compare.h"
 
 namespace BitTorrent
 {
@@ -39,24 +42,33 @@ namespace BitTorrent
 class TransferListSortModel final : public QSortFilterProxyModel
 {
     Q_OBJECT
-    Q_DISABLE_COPY(TransferListSortModel)
+    Q_DISABLE_COPY_MOVE(TransferListSortModel)
 
 public:
     explicit TransferListSortModel(QObject *parent = nullptr);
 
+    void sort(int column, Qt::SortOrder order = Qt::AscendingOrder) override;
+
     void setStatusFilter(TorrentFilter::Type filter);
     void setCategoryFilter(const QString &category);
     void disableCategoryFilter();
     void setTagFilter(const QString &tag);
     void disableTagFilter();
-    void setTrackerFilter(const QSet<BitTorrent::InfoHash> &hashes);
+    void setTrackerFilter(const QSet<BitTorrent::TorrentID> &torrentIDs);
     void disableTrackerFilter();
 
 private:
+    int compare(const QModelIndex &left, const QModelIndex &right) const;
+
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
     bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
     bool matchFilter(int sourceRow, const QModelIndex &sourceParent) const;
-    bool lessThan_impl(const QModelIndex &left, const QModelIndex &right) const;
 
     TorrentFilter m_filter;
+    CachedSettingValue<int> m_subSortColumn;
+    CachedSettingValue<int> m_subSortOrder;
+    int m_lastSortColumn = -1;
+    int m_lastSortOrder = 0;
+
+    Utils::Compare::NaturalCompare<Qt::CaseInsensitive> m_naturalCompare;
 };
