@@ -30,8 +30,10 @@
 
 #include <limits>
 
+#include <QHash>
 #include <QJsonArray>
 #include <QJsonObject>
+#include <QList>
 #include <QSharedPointer>
 
 #include "base/global.h"
@@ -43,8 +45,6 @@
 #include "apierror.h"
 #include "isessionmanager.h"
 
-class SearchPluginManager;
-
 using SearchHandlerPtr = QSharedPointer<SearchHandler>;
 using SearchHandlerDict = QMap<int, SearchHandlerPtr>;
 
@@ -59,6 +59,35 @@ namespace
         if (activeSearches.remove(id))
             session->setData(ACTIVE_SEARCHES, QVariant::fromValue(activeSearches));
     }
+
+    /**
+    * Returns the search categories in JSON format.
+    *
+    * The return value is an array of dictionaries.
+    * The dictionary keys are:
+    *   - "id"
+    *   - "name"
+    */
+    QJsonArray getPluginCategories(QStringList categories)
+    {
+        QJsonArray categoriesInfo
+        {QJsonObject {
+            {QLatin1String("id"), "all"},
+            {QLatin1String("name"), SearchPluginManager::categoryFullName("all")}
+        }};
+
+        categories.sort(Qt::CaseInsensitive);
+        for (const QString &category : categories)
+        {
+            categoriesInfo << QJsonObject
+            {
+                {QLatin1String("id"), category},
+                {QLatin1String("name"), SearchPluginManager::categoryFullName(category)}
+            };
+        }
+
+        return categoriesInfo;
+    }
 }
 
 void SearchController::startAction()
@@ -73,7 +102,8 @@ void SearchController::startAction()
     const QStringList plugins = params()["plugins"].split('|');
 
     QStringList pluginsToUse;
-    if (plugins.size() == 1) {
+    if (plugins.size() == 1)
+    {
         const QString pluginsLower = plugins[0].toLower();
         if (pluginsLower == "all")
             pluginsToUse = SearchPluginManager::instance()->allPlugins();
@@ -82,7 +112,8 @@ void SearchController::startAction()
         else
             pluginsToUse << plugins;
     }
-    else {
+    else
+    {
         pluginsToUse << plugins;
     }
 
@@ -120,7 +151,8 @@ void SearchController::stopAction()
 
     const SearchHandlerPtr searchHandler = searchHandlers[id];
 
-    if (searchHandler->isActive()) {
+    if (searchHandler->isActive())
+    {
         searchHandler->cancelSearch();
         removeActiveSearch(session, id);
     }
@@ -137,9 +169,11 @@ void SearchController::statusAction()
     QJsonArray statusArray;
     const QList<int> searchIds {(id == 0) ? searchHandlers.keys() : QList<int> {id}};
 
-    for (const int searchId : searchIds) {
+    for (const int searchId : searchIds)
+    {
         const SearchHandlerPtr searchHandler = searchHandlers[searchId];
-        statusArray << QJsonObject {
+        statusArray << QJsonObject
+        {
             {"id", searchId},
             {"status", searchHandler->isActive() ? "Running" : "Stopped"},
             {"total", searchHandler->results().size()}
@@ -201,19 +235,6 @@ void SearchController::deleteAction()
     removeActiveSearch(session, id);
 }
 
-void SearchController::categoriesAction()
-{
-    QStringList categories;
-    const QString name = params()["pluginName"].trimmed();
-
-    categories << SearchPluginManager::categoryFullName("all");
-    for (const QString &category : asConst(SearchPluginManager::instance()->getPluginCategories(name)))
-        categories << SearchPluginManager::categoryFullName(category);
-
-    const QJsonArray result = QJsonArray::fromStringList(categories);
-    setResult(result);
-}
-
 void SearchController::pluginsAction()
 {
     const QStringList allPlugins = SearchPluginManager::instance()->allPlugins();
@@ -243,7 +264,7 @@ void SearchController::enablePluginAction()
     requireParams({"names", "enable"});
 
     const QStringList names = params()["names"].split('|');
-    const bool enable = Utils::String::parseBool(params()["enable"].trimmed(), false);
+    const bool enable = Utils::String::parseBool(params()["enable"].trimmed()).value_or(false);
 
     for (const QString &name : names)
         SearchPluginManager::instance()->enablePlugin(name.trimmed(), enable);
@@ -260,7 +281,8 @@ void SearchController::updatePluginsAction()
 
 void SearchController::checkForUpdatesFinished(const QHash<QString, PluginVersion> &updateInfo)
 {
-    if (updateInfo.isEmpty()) {
+    if (updateInfo.isEmpty())
+    {
         LogMsg(tr("All plugins are already up to date."), Log::INFO);
         return;
     }
@@ -268,7 +290,8 @@ void SearchController::checkForUpdatesFinished(const QHash<QString, PluginVersio
     LogMsg(tr("Updating %1 plugins").arg(updateInfo.size()), Log::INFO);
 
     SearchPluginManager *const pluginManager = SearchPluginManager::instance();
-    for (const QString &pluginName : asConst(updateInfo.keys())) {
+    for (const QString &pluginName : asConst(updateInfo.keys()))
+    {
         LogMsg(tr("Updating plugin %1").arg(pluginName), Log::INFO);
         pluginManager->updatePlugin(pluginName);
     }
@@ -317,8 +340,10 @@ int SearchController::generateSearchId() const
 QJsonObject SearchController::getResults(const QList<SearchResult> &searchResults, const bool isSearchActive, const int totalResults) const
 {
     QJsonArray searchResultsArray;
-    for (const SearchResult &searchResult : searchResults) {
-        searchResultsArray << QJsonObject {
+    for (const SearchResult &searchResult : searchResults)
+    {
+        searchResultsArray << QJsonObject
+        {
             {"fileName", searchResult.fileName},
             {"fileUrl", searchResult.fileUrl},
             {"fileSize", searchResult.fileSize},
@@ -329,7 +354,8 @@ QJsonObject SearchController::getResults(const QList<SearchResult> &searchResult
         };
     }
 
-    const QJsonObject result = {
+    const QJsonObject result =
+    {
         {"status", isSearchActive ? "Running" : "Stopped"},
         {"results", searchResultsArray},
         {"total", totalResults}
@@ -355,15 +381,17 @@ QJsonArray SearchController::getPluginsInfo(const QStringList &plugins) const
 {
     QJsonArray pluginsArray;
 
-    for (const QString &plugin : plugins) {
+    for (const QString &plugin : plugins)
+    {
         const PluginInfo *const pluginInfo = SearchPluginManager::instance()->pluginInfo(plugin);
 
-        pluginsArray << QJsonObject {
+        pluginsArray << QJsonObject
+        {
             {"name", pluginInfo->name},
             {"version", QString(pluginInfo->version)},
             {"fullName", pluginInfo->fullName},
             {"url", pluginInfo->url},
-            {"supportedCategories", QJsonArray::fromStringList(pluginInfo->supportedCategories)},
+            {"supportedCategories", getPluginCategories(pluginInfo->supportedCategories)},
             {"enabled", pluginInfo->enabled}
         };
     }
