@@ -33,7 +33,7 @@
 class TransferController : public APIController
 {
     Q_OBJECT
-    Q_DISABLE_COPY(TransferController)
+    Q_DISABLE_COPY_MOVE(TransferController)
 
 public:
     using APIController::APIController;
