@@ -245,7 +245,7 @@ bool GetOuterHwndFromCommandline(HWND&hwndOut)
 	TRACEF("GetOuterHwndFromCommandline:%s|\n",p);
 	if (p && *p++=='/'&&*p++=='U'&&*p++=='A'&&*p++=='C'&&*p++==':') 
 	{
-		hwndOut=(HWND)StrToUInt(p,true);
+		hwndOut=(HWND)StrToUIntPtr(p,true);
 		return (/*IsWindow*/(hwndOut))!=0;
 	}
 	return false;
@@ -283,7 +283,7 @@ BOOL CALLBACK ProcessThreadMessages(HWND hwndDlg=NULL,DWORD*pExitCode=NULL)
 		if (msg.message==WM_QUIT) 
 		{
 			TRACEF("ProcessThreadMessages got WM_QUIT %d:%d, code=%u hDlg=%X\n",GetCurrentProcessId(),GetCurrentThreadId(),msg.wParam,hwndDlg);
-			if (pExitCode)*pExitCode=msg.wParam;
+			if (pExitCode) *pExitCode = (DWORD) msg.wParam;
 			return true;
 		//	ASSERT(!"TODO: should we handle WM_QUIT?");
 		//	ec=ERROR_CANCELLED;
@@ -461,7 +461,7 @@ LRESULT CALLBACK OuterWndProc(HWND hwnd,UINT msg,WPARAM wp,LPARAM lp)
 			break;
 #ifdef FEAT_AUTOPAGEJUMP
 		case SOSI_ELEVATIONPAGE_GUIINITERRORCODE:
-			OG().ecInner_InitSharedData=lp;
+			OG().ecInner_InitSharedData = (DWORD) lp;
 			break;
 #endif
 		}
@@ -630,7 +630,7 @@ LRESULT CALLBACK OuterMainWndSubProc(HWND hwnd,UINT msg,WPARAM wp,LPARAM lp)
 	switch(msg)
 	{
 	case WM_NOTIFY_OUTER_NEXT:
-		G().JumpPageOffset+=wp;
+		G().JumpPageOffset += (WORD) wp;
 		TRACEF("WM_NOTIFY_OUTER_NEXT: now=%d change=%d\n",G().JumpPageOffset,wp);
 		break;
 	}
@@ -814,7 +814,7 @@ extern "C" void __declspec(dllexport) __cdecl _(HWND hwndNSIS,UINT NSIScchStr,NS
 #endif
 					
 					p=FindExePathEnd(pCL);
-					len=(p-pCL)/sizeof(TCHAR);
+					len=(UINT)(p-pCL) / sizeof(TCHAR);
 					ec=ERROR_FILE_NOT_FOUND;
 
 					if (p && len)
